from typing import Any
from typing import Callable
from typing import Dict
from typing import Generic
from typing import Iterator
from typing import List
from typing import Optional
from typing import overload
from typing import Sequence
from typing import Set
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union

from . import compat

_T = TypeVar("_T")
_F = TypeVar("_F", bound=Callable[..., Any])
_MP = TypeVar("_MP", bound=memoized_property[Any])
_MA = TypeVar("_MA", bound=HasMemoized.memoized_attribute[Any])
_HP = TypeVar("_HP", bound=hybridproperty)
_HM = TypeVar("_HM", bound=hybridmethod)

_LoaderType = Callable[[], _T]

def md5_hex(x: Any) -> str: ...

class safe_reraise:
    warn_only: bool = ...
    def __init__(self, warn_only: bool = ...) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type_: Any, value: Any, traceback: Any) -> None: ...

def walk_subclasses(cls: Type[Any]) -> Iterator[Type[Any]]: ...
def string_or_unprintable(element: Any) -> str: ...
def clsname_as_plain_name(cls: Type[Any]) -> str: ...
def method_is_overridden(
    instance_or_cls: Any, against_method: Callable[..., Any]
) -> bool: ...
def decode_slice(slc: slice) -> Tuple[Any, ...]: ...
def map_bits(fn: Callable[[int], _T], n: Any) -> Iterator[_T]: ...
def decorator(target: Any) -> Any: ...
def public_factory(
    target: Any, location: Any, class_location: Optional[Any] = ...
) -> Any: ...

class PluginLoader(Generic[_T]):
    group: str = ...
    impls: Dict[str, Union[_LoaderType[_T], _T]] = ...
    auto_fn: Optional[Callable[[str], Optional[_T]]] = ...
    def __init__(
        self,
        group: str,
        auto_fn: Optional[Callable[[str], Optional[_T]]] = ...,
    ) -> None: ...
    def clear(self) -> None: ...
    def load(self, name: str) -> _T: ...
    def register(self, name: str, modulepath: str, objname: str) -> None: ...

def get_cls_kwargs(
    cls: Type[Any], _set: Optional[Set[str]] = ...
) -> Set[str]: ...
def get_func_kwargs(func: Any) -> List[str]: ...
def get_callable_argspec(
    fn: Callable[..., Any], no_self: bool = ..., _is_init: bool = ...
) -> compat.FullArgSpec: ...
def format_argspec_plus(fn: Any, grouped: bool = ...) -> Any: ...
def format_argspec_init(method: Any, grouped: bool = ...) -> Any: ...
def create_proxy_methods(
    target_cls: Any,
    target_cls_sphinx_name: Any,
    proxy_cls_sphinx_name: Any,
    classmethods: Any = ...,
    methods: Any = ...,
    attributes: Any = ...,
) -> Any: ...
def getargspec_init(
    method: Any,
) -> Union[
    compat.FullArgSpec,
    Tuple[List[str], Optional[str], Optional[str], Optional[str]],
]: ...
def unbound_method_to_callable(func_or_cls: Any) -> Any: ...
def generic_repr(
    obj: Any,
    additional_kw: Any = ...,
    to_inspect: Optional[Any] = ...,
    omit_kwarg: Any = ...,
) -> str: ...

class portable_instancemethod:
    target: Any = ...
    name: Any = ...
    kwargs: Any = ...
    def __init__(self, meth: Any, kwargs: Tuple[Any, ...] = ...) -> None: ...
    def __call__(self, *arg: Any, **kw: Any) -> Any: ...

def class_hierarchy(cls: Type[Any]) -> List[Type[Any]]: ...
def iterate_attributes(cls: Type[Any]) -> Iterator[Tuple[str, Any]]: ...
def monkeypatch_proxied_specials(
    into_cls: Any,
    from_cls: Any,
    skip: Optional[Any] = ...,
    only: Optional[Any] = ...,
    name: str = ...,
    from_instance: Optional[Any] = ...,
) -> None: ...
def methods_equivalent(meth1: Any, meth2: Any) -> bool: ...
def as_interface(
    obj: Any,
    cls: Optional[Any] = ...,
    methods: Optional[Any] = ...,
    required: Optional[Any] = ...,
) -> Any: ...

class memoized_property(Generic[_T]):
    fget: Callable[..., _T] = ...
    __doc__: Optional[str] = ...
    __name__: str = ...
    def __init__(
        self, fget: Callable[..., _T], doc: Optional[str] = ...
    ) -> None: ...
    @overload
    def __get__(self: _MP, obj: None, cls: Any) -> _MP: ...
    @overload
    def __get__(self, obj: Any, cls: Any) -> _T: ...
    @classmethod
    def reset(cls, obj: Any, name: str) -> None: ...

def memoized_instancemethod(fn: Any) -> Any: ...

class HasMemoized:
    class memoized_attribute(Generic[_T]):
        fget: Callable[..., _T] = ...
        __doc__: Optional[str] = ...
        __name__: str = ...
        def __init__(
            self, fget: Callable[..., _T], doc: Optional[str] = ...
        ) -> None: ...
        @overload
        def __get__(self: _MA, obj: None, cls: Any) -> _MA: ...
        @overload
        def __get__(self, obj: Any, cls: Any) -> _T: ...
    @classmethod
    def memoized_instancemethod(cls, fn: Any) -> Any: ...

class MemoizedSlots:
    def __getattr__(self, key: Any) -> Any: ...

def asbool(obj: Any) -> bool: ...
def bool_or_str(*text: Any) -> Callable[[Any], Any]: ...
def asint(value: Any) -> Optional[int]: ...
def coerce_kw_type(
    kw: Dict[str, Any],
    key: str,
    type_: Type[Any],
    flexi_bool: bool = ...,
    dest: Optional[Dict[str, Any]] = ...,
) -> None: ...
def constructor_key(obj: Any, cls: Type[Any]) -> Tuple[Any, ...]: ...
def constructor_copy(obj: Any, cls: Type[_T], *args: Any, **kw: Any) -> _T: ...
def counter() -> Callable[[], Iterator[int]]: ...
def duck_type_collection(
    specimen: Any, default: Optional[Type[Any]] = ...
) -> Optional[Type[Any]]: ...
def assert_arg_type(arg: _T, argtype: Type[Any], name: str) -> _T: ...
def dictlike_iteritems(dictlike: Any) -> Iterator[Tuple[Any, Any]]: ...

class classproperty(property, Generic[_T]):
    __doc__: Optional[str] = ...
    def __init__(
        self, fget: Callable[[Any], _T], *arg: Any, **kw: Any
    ) -> None: ...
    def __get__(desc: Any, self: Any, cls: Any) -> _T: ...  # type: ignore[override]

class hybridproperty:
    func: Any = ...
    clslevel: Any = ...
    def __init__(self, func: Any) -> None: ...
    @overload
    def __get__(self: _HP, instance: None, owner: Any) -> _HP: ...
    @overload
    def __get__(self, instance: Any, owner: Any) -> Any: ...
    def classlevel(self: _HP, func: Any) -> _HP: ...

class hybridmethod:
    func: Any = ...
    clslevel: Any = ...
    def __init__(self, func: Any) -> None: ...
    def __get__(self, instance: Any, owner: Any) -> Any: ...
    def classlevel(self: _HM, func: Any) -> _HM: ...

class _symbol(int):
    def __new__(
        self,
        name: Any,
        doc: Optional[Any] = ...,
        canonical: Optional[Any] = ...,
    ) -> _symbol: ...
    def __reduce__(self) -> Any: ...

class symbol:
    symbols: Any = ...
    def __new__(  # type: ignore[misc]
        cls,
        name: Any,
        doc: Optional[Any] = ...,
        canonical: Optional[Any] = ...,
    ) -> _symbol: ...
    @classmethod
    def parse_user_argument(
        cls,
        arg: Any,
        choices: Any,
        name: Any,
        resolve_symbol_names: bool = ...,
    ) -> Optional[_symbol]: ...

def set_creation_order(instance: Any) -> None: ...
def warn_exception(
    func: Callable[..., _T], *args: Any, **kwargs: Any
) -> _T: ...
def ellipses_string(
    value: compat.text_type, len_: int = ...
) -> compat.text_type: ...
def warn(msg: str) -> None: ...
def warn_limited(msg: str, args: Any) -> None: ...
def only_once(fn: _F, retry_on_exception: bool) -> _F: ...
def chop_traceback(
    tb: Any, exclude_prefix: Any = ..., exclude_suffix: Any = ...
) -> str: ...

NoneType: Type[None]

def attrsetter(attrname: str) -> Any: ...

class EnsureKWArgType(type):
    def __init__(cls, clsname: Any, bases: Any, clsdict: Any) -> None: ...

def wrap_callable(wrapper: Any, fn: Any) -> Any: ...
def quoted_token_parser(value: str) -> List[str]: ...
def add_parameter_text(params: Any, text: str) -> Callable[[_F], _F]: ...
def inject_docstring_text(doctext: str, injecttext: str, pos: int) -> str: ...
def inject_param_text(doctext: str, inject_params: Any) -> str: ...
def repr_tuple_names(names: Sequence[str]) -> str: ...
def has_compiled_ext() -> bool: ...



from .api import CANCEL as CANCEL
from .api import contains as contains
from .api import listen as listen
from .api import listens_for as listens_for
from .api import NO_RETVAL as NO_RETVAL
from .api import remove as remove
from .attr import RefCollection as RefCollection
from .base import dispatcher as dispatcher
from .base import Events as Events



from typing import Any
from typing import Optional

from .base import ENUM as ENUM
from .base import INTERVAL as INTERVAL
from .base import PGCompiler as PGCompiler
from .base import PGDialect as PGDialect
from .base import PGExecutionContext as PGExecutionContext
from .base import PGIdentifierPreparer as PGIdentifierPreparer
from .base import UUID as UUID
from .json import JSON as JSON
from .json import JSONB as JSONB
from .json import JSONPathType as JSONPathType
from ... import exc as exc
from ... import processors as processors
from ... import types as sqltypes
from ... import util as util
from ...sql.elements import quoted_name as quoted_name

class _PGNumeric(sqltypes.Numeric):
    def result_processor(self, dialect: Any, coltype: Any): ...

class _PGNumericNoBind(_PGNumeric):
    def bind_processor(self, dialect: Any) -> None: ...

class _PGJSON(JSON):
    def result_processor(self, dialect: Any, coltype: Any) -> None: ...
    def get_dbapi_type(self, dbapi: Any): ...

class _PGJSONB(JSONB):
    def result_processor(self, dialect: Any, coltype: Any) -> None: ...
    def get_dbapi_type(self, dbapi: Any): ...

class _PGJSONIndexType(sqltypes.JSON.JSONIndexType):
    def get_dbapi_type(self, dbapi: Any) -> None: ...

class _PGJSONIntIndexType(sqltypes.JSON.JSONIntIndexType):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGJSONStrIndexType(sqltypes.JSON.JSONStrIndexType):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGJSONPathType(JSONPathType):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGUUID(UUID):
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class _PGEnum(ENUM):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGInterval(INTERVAL):
    def get_dbapi_type(self, dbapi: Any): ...
    @classmethod
    def adapt_emulated_to_native(cls, interval: Any, **kw: Any): ...

class _PGTimeStamp(sqltypes.DateTime):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGTime(sqltypes.Time):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGInteger(sqltypes.Integer):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGSmallInteger(sqltypes.SmallInteger):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGNullType(sqltypes.NullType):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGBigInteger(sqltypes.BigInteger):
    def get_dbapi_type(self, dbapi: Any): ...

class _PGBoolean(sqltypes.Boolean):
    def get_dbapi_type(self, dbapi: Any): ...

class PGExecutionContext_pg8000(PGExecutionContext):
    def pre_exec(self) -> None: ...

class PGCompiler_pg8000(PGCompiler):
    def visit_mod_binary(self, binary: Any, operator: Any, **kw: Any): ...

class PGIdentifierPreparer_pg8000(PGIdentifierPreparer):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class PGDialect_pg8000(PGDialect):
    driver: str = ...
    supports_unicode_statements: bool = ...
    supports_unicode_binds: bool = ...
    default_paramstyle: str = ...
    supports_sane_multi_rowcount: bool = ...
    execution_ctx_cls: Any = ...
    statement_compiler: Any = ...
    preparer: Any = ...
    use_setinputsizes: bool = ...
    description_encoding: Any = ...
    colspecs: Any = ...
    client_encoding: Any = ...
    def __init__(
        self, client_encoding: Optional[Any] = ..., **kwargs: Any
    ) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(self, url: Any): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...
    def set_isolation_level(self, connection: Any, level: Any) -> None: ...
    def set_readonly(self, connection: Any, value: Any) -> None: ...
    def get_readonly(self, connection: Any): ...
    def set_deferrable(self, connection: Any, value: Any) -> None: ...
    def get_deferrable(self, connection: Any): ...
    def set_client_encoding(
        self, connection: Any, client_encoding: Any
    ) -> None: ...
    def do_set_input_sizes(
        self, cursor: Any, list_of_tuples: Any, context: Any
    ) -> None: ...
    def do_begin_twophase(self, connection: Any, xid: Any) -> None: ...
    def do_prepare_twophase(self, connection: Any, xid: Any) -> None: ...
    def do_rollback_twophase(
        self,
        connection: Any,
        xid: Any,
        is_prepared: bool = ...,
        recover: bool = ...,
    ) -> None: ...
    def do_commit_twophase(
        self,
        connection: Any,
        xid: Any,
        is_prepared: bool = ...,
        recover: bool = ...,
    ) -> None: ...
    def do_recover_twophase(self, connection: Any): ...
    def on_connect(self): ...

dialect = PGDialect_pg8000



import sys
from typing import Any
from typing import NoReturn

have_greenlet: bool

if sys.version_info >= (3, 0):
    from ._concurrency_py3k import AsyncAdaptedLock as AsyncAdaptedLock
    from ._concurrency_py3k import asyncio as asyncio
    from ._concurrency_py3k import await_fallback as await_fallback
    from ._concurrency_py3k import await_only as await_only
    from ._concurrency_py3k import greenlet_spawn as greenlet_spawn
else:
    asyncio: None
    def await_only(thing: Any) -> Any: ...
    def await_fallback(thing: Any) -> Any: ...
    def greenlet_spawn(fn: Any, *args: Any, **kw: Any) -> NoReturn: ...
    def AsyncAdaptedLock(*args: Any, **kw: Any) -> NoReturn: ...



from typing import Any
from typing import Optional

from .pymysql import MySQLDialect_pymysql as MySQLDialect_pymysql
from ... import pool as pool
from ... import util as util
from ...util.concurrency import await_fallback as await_fallback
from ...util.concurrency import await_only as await_only

class AsyncAdapt_aiomysql_cursor:
    server_side: bool = ...
    await_: Any = ...
    def __init__(self, adapt_connection: Any) -> None: ...
    @property
    def description(self): ...
    @property
    def rowcount(self): ...
    @property
    def arraysize(self): ...
    @arraysize.setter
    def arraysize(self, value: Any) -> None: ...
    @property
    def lastrowid(self): ...
    def close(self) -> None: ...
    def execute(self, operation: Any, parameters: Optional[Any] = ...): ...
    def executemany(self, operation: Any, seq_of_parameters: Any): ...
    def setinputsizes(self, *inputsizes: Any) -> None: ...
    def __iter__(self) -> Any: ...
    def fetchone(self): ...
    def fetchmany(self, size: Optional[Any] = ...): ...
    def fetchall(self): ...

class AsyncAdapt_aiomysql_ss_cursor(AsyncAdapt_aiomysql_cursor):
    server_side: bool = ...
    await_: Any = ...
    def __init__(self, adapt_connection: Any) -> None: ...
    def close(self) -> None: ...
    def fetchone(self): ...
    def fetchmany(self, size: Optional[Any] = ...): ...
    def fetchall(self): ...

class AsyncAdapt_aiomysql_connection:
    await_: Any = ...
    dbapi: Any = ...
    def __init__(self, dbapi: Any, connection: Any) -> None: ...
    def ping(self, reconnect: Any): ...
    def character_set_name(self): ...
    def autocommit(self, value: Any) -> None: ...
    def cursor(self, server_side: bool = ...): ...
    def rollback(self) -> None: ...
    def commit(self) -> None: ...
    def close(self) -> None: ...

class AsyncAdaptFallback_aiomysql_connection(AsyncAdapt_aiomysql_connection):
    await_: Any = ...

class AsyncAdapt_aiomysql_dbapi:
    aiomysql: Any = ...
    pymysql: Any = ...
    paramstyle: str = ...
    def __init__(self, aiomysql: Any, pymysql: Any) -> None: ...
    def connect(self, *arg: Any, **kw: Any): ...

class MySQLDialect_aiomysql(MySQLDialect_pymysql):
    driver: str = ...
    supports_server_side_cursors: bool = ...
    is_async: bool = ...
    @classmethod
    def dbapi(cls): ...
    @classmethod
    def get_pool_class(cls, url: Any): ...
    def create_connect_args(self, url: Any): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...

dialect = MySQLDialect_aiomysql



from typing import Any
from typing import Optional

from ... import exc as exc
from ... import schema as schema
from ... import sql as sql
from ... import util as util
from ...engine import characteristics as characteristics
from ...engine import default as default
from ...engine import reflection as reflection
from ...sql import coercions as coercions
from ...sql import compiler as compiler
from ...sql import elements as elements
from ...sql import expression as expression
from ...sql import roles as roles
from ...sql import sqltypes as sqltypes
from ...sql.ddl import DDLBase as DDLBase
from ...types import BIGINT as BIGINT
from ...types import BOOLEAN as BOOLEAN
from ...types import CHAR as CHAR
from ...types import DATE as DATE
from ...types import FLOAT as FLOAT
from ...types import INTEGER as INTEGER
from ...types import NUMERIC as NUMERIC
from ...types import REAL as REAL
from ...types import SMALLINT as SMALLINT
from ...types import TEXT as TEXT
from ...types import VARCHAR as VARCHAR

IDX_USING: Any
AUTOCOMMIT_REGEXP: Any
RESERVED_WORDS: Any

class BYTEA(sqltypes.LargeBinary):
    __visit_name__: str = ...

class DOUBLE_PRECISION(sqltypes.Float):
    __visit_name__: str = ...

class INET(sqltypes.TypeEngine):
    __visit_name__: str = ...

PGInet = INET

class CIDR(sqltypes.TypeEngine):
    __visit_name__: str = ...

PGCidr = CIDR

class MACADDR(sqltypes.TypeEngine):
    __visit_name__: str = ...

PGMacAddr = MACADDR

class MONEY(sqltypes.TypeEngine):
    __visit_name__: str = ...

class OID(sqltypes.TypeEngine):
    __visit_name__: str = ...

class REGCLASS(sqltypes.TypeEngine):
    __visit_name__: str = ...

class TIMESTAMP(sqltypes.TIMESTAMP):
    precision: Any = ...
    def __init__(
        self, timezone: bool = ..., precision: Optional[Any] = ...
    ) -> None: ...

class TIME(sqltypes.TIME):
    precision: Any = ...
    def __init__(
        self, timezone: bool = ..., precision: Optional[Any] = ...
    ) -> None: ...

class INTERVAL(sqltypes.NativeForEmulated, sqltypes._AbstractInterval):
    __visit_name__: str = ...
    native: bool = ...
    precision: Any = ...
    fields: Any = ...
    def __init__(
        self, precision: Optional[Any] = ..., fields: Optional[Any] = ...
    ) -> None: ...
    @classmethod
    def adapt_emulated_to_native(cls, interval: Any, **kw: Any): ...
    def as_generic(self, allow_nulltype: bool = ...): ...
    @property
    def python_type(self): ...

PGInterval = INTERVAL

class BIT(sqltypes.TypeEngine):
    __visit_name__: str = ...
    length: Any = ...
    varying: Any = ...
    def __init__(
        self, length: Optional[Any] = ..., varying: bool = ...
    ) -> None: ...

PGBit = BIT

class UUID(sqltypes.TypeEngine):
    __visit_name__: str = ...
    as_uuid: Any = ...
    def __init__(self, as_uuid: bool = ...) -> None: ...
    def coerce_compared_value(self, op: Any, value: Any): ...
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

PGUuid = UUID

class TSVECTOR(sqltypes.TypeEngine):
    __visit_name__: str = ...

class ENUM(sqltypes.NativeForEmulated, sqltypes.Enum):
    native_enum: bool = ...
    create_type: Any = ...
    def __init__(self, *enums: Any, **kw: Any) -> None: ...
    @classmethod
    def adapt_emulated_to_native(cls, impl: Any, **kw: Any): ...
    def create(
        self, bind: Optional[Any] = ..., checkfirst: bool = ...
    ) -> None: ...
    def drop(
        self, bind: Optional[Any] = ..., checkfirst: bool = ...
    ) -> None: ...
    class EnumGenerator(DDLBase):
        checkfirst: Any = ...
        def __init__(
            self,
            dialect: Any,
            connection: Any,
            checkfirst: bool = ...,
            **kwargs: Any,
        ) -> None: ...
        def visit_enum(self, enum: Any) -> None: ...
    class EnumDropper(DDLBase):
        checkfirst: Any = ...
        def __init__(
            self,
            dialect: Any,
            connection: Any,
            checkfirst: bool = ...,
            **kwargs: Any,
        ) -> None: ...
        def visit_enum(self, enum: Any) -> None: ...

colspecs: Any
ischema_names: Any

class PGCompiler(compiler.SQLCompiler):
    def visit_array(self, element: Any, **kw: Any): ...
    def visit_slice(self, element: Any, **kw: Any): ...
    def visit_json_getitem_op_binary(
        self, binary: Any, operator: Any, _cast_applied: bool = ..., **kw: Any
    ): ...
    def visit_json_path_getitem_op_binary(
        self, binary: Any, operator: Any, _cast_applied: bool = ..., **kw: Any
    ): ...
    def visit_getitem_binary(self, binary: Any, operator: Any, **kw: Any): ...
    def visit_aggregate_order_by(self, element: Any, **kw: Any): ...
    def visit_match_op_binary(self, binary: Any, operator: Any, **kw: Any): ...
    def visit_ilike_op_binary(self, binary: Any, operator: Any, **kw: Any): ...
    def visit_not_ilike_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_regexp_match_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_not_regexp_match_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_regexp_replace_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_empty_set_expr(self, element_types: Any): ...
    def render_literal_value(self, value: Any, type_: Any): ...
    def visit_sequence(self, seq: Any, **kw: Any): ...
    def limit_clause(self, select: Any, **kw: Any): ...
    def format_from_hint_text(
        self, sqltext: Any, table: Any, hint: Any, iscrud: Any
    ): ...
    def get_select_precolumns(self, select: Any, **kw: Any): ...
    def for_update_clause(self, select: Any, **kw: Any): ...
    def returning_clause(self, stmt: Any, returning_cols: Any): ...
    def visit_substring_func(self, func: Any, **kw: Any): ...
    def visit_on_conflict_do_nothing(self, on_conflict: Any, **kw: Any): ...
    def visit_on_conflict_do_update(self, on_conflict: Any, **kw: Any): ...
    def update_from_clause(
        self,
        update_stmt: Any,
        from_table: Any,
        extra_froms: Any,
        from_hints: Any,
        **kw: Any,
    ): ...
    def delete_extra_from_clause(
        self,
        delete_stmt: Any,
        from_table: Any,
        extra_froms: Any,
        from_hints: Any,
        **kw: Any,
    ): ...
    def fetch_clause(self, select: Any, **kw: Any): ...

class PGDDLCompiler(compiler.DDLCompiler):
    def get_column_specification(self, column: Any, **kwargs: Any): ...
    def visit_check_constraint(self, constraint: Any): ...
    def visit_drop_table_comment(self, drop: Any): ...
    def visit_create_enum_type(self, create: Any): ...
    def visit_drop_enum_type(self, drop: Any): ...
    def visit_create_index(self, create: Any): ...
    def visit_drop_index(self, drop: Any): ...
    def visit_exclude_constraint(self, constraint: Any, **kw: Any): ...
    def post_create_table(self, table: Any): ...
    def visit_computed_column(self, generated: Any): ...
    def visit_create_sequence(self, create: Any, **kw: Any): ...

class PGTypeCompiler(compiler.GenericTypeCompiler):
    def visit_TSVECTOR(self, type_: Any, **kw: Any): ...
    def visit_INET(self, type_: Any, **kw: Any): ...
    def visit_CIDR(self, type_: Any, **kw: Any): ...
    def visit_MACADDR(self, type_: Any, **kw: Any): ...
    def visit_MONEY(self, type_: Any, **kw: Any): ...
    def visit_OID(self, type_: Any, **kw: Any): ...
    def visit_REGCLASS(self, type_: Any, **kw: Any): ...
    def visit_FLOAT(self, type_: Any, **kw: Any): ...
    def visit_DOUBLE_PRECISION(self, type_: Any, **kw: Any): ...
    def visit_BIGINT(self, type_: Any, **kw: Any): ...
    def visit_HSTORE(self, type_: Any, **kw: Any): ...
    def visit_JSON(self, type_: Any, **kw: Any): ...
    def visit_JSONB(self, type_: Any, **kw: Any): ...
    def visit_INT4RANGE(self, type_: Any, **kw: Any): ...
    def visit_INT8RANGE(self, type_: Any, **kw: Any): ...
    def visit_NUMRANGE(self, type_: Any, **kw: Any): ...
    def visit_DATERANGE(self, type_: Any, **kw: Any): ...
    def visit_TSRANGE(self, type_: Any, **kw: Any): ...
    def visit_TSTZRANGE(self, type_: Any, **kw: Any): ...
    def visit_datetime(self, type_: Any, **kw: Any): ...
    def visit_enum(self, type_: Any, **kw: Any): ...
    def visit_ENUM(
        self, type_: Any, identifier_preparer: Optional[Any] = ..., **kw: Any
    ): ...
    def visit_TIMESTAMP(self, type_: Any, **kw: Any): ...
    def visit_TIME(self, type_: Any, **kw: Any): ...
    def visit_INTERVAL(self, type_: Any, **kw: Any): ...
    def visit_BIT(self, type_: Any, **kw: Any): ...
    def visit_UUID(self, type_: Any, **kw: Any): ...
    def visit_large_binary(self, type_: Any, **kw: Any): ...
    def visit_BYTEA(self, type_: Any, **kw: Any): ...
    def visit_ARRAY(self, type_: Any, **kw: Any): ...

class PGIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any = ...
    def format_type(self, type_: Any, use_schema: bool = ...): ...

class PGInspector(reflection.Inspector):
    def get_table_oid(self, table_name: Any, schema: Optional[Any] = ...): ...
    def get_enums(self, schema: Optional[Any] = ...): ...
    def get_foreign_table_names(self, schema: Optional[Any] = ...): ...
    def get_view_names(
        self, schema: Optional[Any] = ..., include: Any = ...
    ): ...

class CreateEnumType(schema._CreateDropBase):
    __visit_name__: str = ...

class DropEnumType(schema._CreateDropBase):
    __visit_name__: str = ...

class PGExecutionContext(default.DefaultExecutionContext):
    def fire_sequence(self, seq: Any, type_: Any): ...
    def get_insert_default(self, column: Any): ...
    def should_autocommit_text(self, statement: Any): ...

class PGReadOnlyConnectionCharacteristic(
    characteristics.ConnectionCharacteristic
):
    transactional: bool = ...
    def reset_characteristic(self, dialect: Any, dbapi_conn: Any) -> None: ...
    def set_characteristic(
        self, dialect: Any, dbapi_conn: Any, value: Any
    ) -> None: ...
    def get_characteristic(self, dialect: Any, dbapi_conn: Any): ...

class PGDeferrableConnectionCharacteristic(
    characteristics.ConnectionCharacteristic
):
    transactional: bool = ...
    def reset_characteristic(self, dialect: Any, dbapi_conn: Any) -> None: ...
    def set_characteristic(
        self, dialect: Any, dbapi_conn: Any, value: Any
    ) -> None: ...
    def get_characteristic(self, dialect: Any, dbapi_conn: Any): ...

class PGDialect(default.DefaultDialect):
    name: str = ...
    supports_alter: bool = ...
    max_identifier_length: int = ...
    supports_sane_rowcount: bool = ...
    supports_native_enum: bool = ...
    supports_native_boolean: bool = ...
    supports_smallserial: bool = ...
    supports_sequences: bool = ...
    sequences_optional: bool = ...
    preexecute_autoincrement_sequences: bool = ...
    postfetch_lastrowid: bool = ...
    supports_comments: bool = ...
    supports_default_values: bool = ...
    supports_empty_insert: bool = ...
    supports_multivalues_insert: bool = ...
    default_paramstyle: str = ...
    ischema_names: Any = ...
    colspecs: Any = ...
    statement_compiler: Any = ...
    ddl_compiler: Any = ...
    type_compiler: Any = ...
    preparer: Any = ...
    execution_ctx_cls: Any = ...
    inspector: Any = ...
    isolation_level: Any = ...
    implicit_returning: bool = ...
    full_returning: bool = ...
    connection_characteristics: Any = ...
    construct_arguments: Any = ...
    reflection_options: Any = ...
    def __init__(
        self,
        isolation_level: Optional[Any] = ...,
        json_serializer: Optional[Any] = ...,
        json_deserializer: Optional[Any] = ...,
        **kwargs: Any,
    ) -> None: ...
    def initialize(self, connection: Any) -> None: ...
    def on_connect(self): ...
    def set_isolation_level(self, connection: Any, level: Any) -> None: ...
    def get_isolation_level(self, connection: Any): ...
    def set_readonly(self, connection: Any, value: Any) -> None: ...
    def get_readonly(self, connection: Any) -> None: ...
    def set_deferrable(self, connection: Any, value: Any) -> None: ...
    def get_deferrable(self, connection: Any) -> None: ...
    def do_begin_twophase(self, connection: Any, xid: Any) -> None: ...
    def do_prepare_twophase(self, connection: Any, xid: Any) -> None: ...
    def do_rollback_twophase(
        self,
        connection: Any,
        xid: Any,
        is_prepared: bool = ...,
        recover: bool = ...,
    ) -> None: ...
    def do_commit_twophase(
        self,
        connection: Any,
        xid: Any,
        is_prepared: bool = ...,
        recover: bool = ...,
    ) -> None: ...
    def do_recover_twophase(self, connection: Any): ...
    def has_schema(self, connection: Any, schema: Any): ...
    def has_table(
        self, connection: Any, table_name: Any, schema: Optional[Any] = ...
    ): ...
    def has_sequence(
        self, connection: Any, sequence_name: Any, schema: Optional[Any] = ...
    ): ...
    def has_type(
        self, connection: Any, type_name: Any, schema: Optional[Any] = ...
    ): ...
    def get_table_oid(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_schema_names(self, connection: Any, **kw: Any): ...
    def get_table_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_view_names(
        self,
        connection: Any,
        schema: Optional[Any] = ...,
        include: Any = ...,
        **kw: Any,
    ): ...
    def get_sequence_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_view_definition(
        self,
        connection: Any,
        view_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_columns(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_pk_constraint(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_foreign_keys(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        postgresql_ignore_search_path: bool = ...,
        **kw: Any,
    ): ...
    def get_indexes(
        self, connection: Any, table_name: Any, schema: Any, **kw: Any
    ): ...
    def get_unique_constraints(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_table_comment(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_check_constraints(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...



from typing import Any
from typing import Generic
from typing import Mapping
from typing import Optional
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union

from typing_extensions import Protocol

from . import operators
from .base import SchemaEventTarget
from .elements import ClauseElement
from .sqltypes import Boolean
from .sqltypes import Indexable
from .sqltypes import Integer
from .sqltypes import MatchType
from .sqltypes import NullType
from .sqltypes import String
from .sqltypes import TableValueType
from .visitors import Traversible
from .visitors import TraversibleType
from .. import util
from ..engine import Dialect
from ..util import compat

BOOLEANTYPE: Boolean
INTEGERTYPE: Integer
NULLTYPE: NullType
STRINGTYPE: String
MATCHTYPE: MatchType
INDEXABLE = Indexable
TABLEVALUE: TableValueType

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)
_T_contra = TypeVar("_T_contra", contravariant=True)
_U = TypeVar("_U")

_TE = TypeVar("_TE", bound=TypeEngine[Any])
_NFE = TypeVar("_NFE", bound=NativeForEmulated)
_UDT = TypeVar("_UDT", bound=UserDefinedType[Any])
_TD = TypeVar("_TD", bound=TypeDecorator[Any])
_VT = TypeVar("_VT", bound=Variant[Any])

class _LiteralProcessor(Protocol[_T_contra]):
    def __call__(self, __value: Optional[_T_contra]) -> str: ...

class _BindProcessor(Protocol[_T_contra]):
    def __call__(self, __value: Optional[_T_contra]) -> Optional[Any]: ...

class _ResultProcessor(Protocol[_T_co]):
    def __call__(self, __value: Optional[Any]) -> Optional[_T_co]: ...

class TypeEngine(Traversible, Generic[_T]):
    class Comparator(operators.ColumnOperators, Generic[_TE]):
        default_comparator: Any = ...
        def __clause_element__(self) -> ClauseElement: ...
        expr: ClauseElement = ...
        type: _TE = ...
        def __init__(self, expr: ClauseElement) -> None: ...
        def operate(
            self, op: Any, *other: Any, **kwargs: Any
        ) -> ClauseElement: ...
        def reverse_operate(
            self, op: Any, other: Any, **kwargs: Any
        ) -> ClauseElement: ...
        def __reduce__(self) -> Any: ...
    hashable: bool = ...
    comparator_factory: Type[Comparator[TypeEngine[_T]]] = ...
    sort_key_function: Optional[compat._SortKeyFunction] = ...
    should_evaluate_none: bool = ...
    def evaluates_none(self: _TE) -> _TE: ...
    def copy(self: _TE, **kw: Any) -> _TE: ...
    def compare_against_backend(
        self, dialect: Dialect, conn_type: Any
    ) -> Any: ...
    def copy_value(self, value: _T) -> _T: ...
    def literal_processor(
        self, dialect: Dialect
    ) -> Optional[_LiteralProcessor[_T]]: ...
    def bind_processor(
        self, dialect: Dialect
    ) -> Optional[_BindProcessor[_T]]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> Optional[_ResultProcessor[_T]]: ...
    def column_expression(self, colexpr: Any) -> Any: ...
    def bind_expression(self, bindvalue: Any) -> Any: ...
    def compare_values(self, x: Any, y: Any) -> bool: ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...
    @property
    def python_type(self) -> Type[_T]: ...
    def with_variant(
        self, type_: Type[TypeEngine[_U]], dialect_name: str
    ) -> Variant[_U]: ...
    def as_generic(self, allow_nulltype: bool = ...) -> TypeEngine[Any]: ...
    def dialect_impl(self, dialect: Dialect) -> Type[Any]: ...
    def adapt(self, __cls: Type[_U], **kw: Any) -> _U: ...
    def coerce_compared_value(
        self, op: Any, value: Any
    ) -> TypeEngine[Any]: ...
    def compile(self, dialect: Optional[Dialect] = ...) -> Any: ...

class VisitableCheckKWArg(util.EnsureKWArgType, TraversibleType): ...

class UserDefinedType(TypeEngine[_T], metaclass=VisitableCheckKWArg):
    __visit_name__: str = ...
    ensure_kwarg: str = ...
    def coerce_compared_value(self: _UDT, op: Any, value: Any) -> _UDT: ...

class Emulated:
    def adapt_to_emulated(self, impltype: Any, **kw: Any) -> Any: ...
    def adapt(self, __impltype: Any, **kw: Any) -> Any: ...

class NativeForEmulated:
    @classmethod
    def adapt_native_to_emulated(cls, impl: Any, **kw: Any) -> Any: ...
    @classmethod
    def adapt_emulated_to_native(
        cls: Type[_NFE], impl: Any, **kw: Any
    ) -> _NFE: ...

class TypeDecorator(SchemaEventTarget, TypeEngine[_T]):
    __visit_name__: str = ...
    impl: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    coerce_to_is_types: Tuple[Type[Any], ...] = ...
    class Comparator(TypeEngine.Comparator[_TE]):
        def operate(
            self, op: Any, *other: Any, **kwargs: Any
        ) -> ClauseElement: ...
        def reverse_operate(
            self, op: Any, other: Any, **kwargs: Any
        ) -> ClauseElement: ...
    @property
    def comparator_factory(self) -> Type[Comparator[TypeEngine[_T]]]: ...  # type: ignore[override]
    def type_engine(self, dialect: Dialect) -> TypeEngine[Any]: ...
    def load_dialect_impl(self, dialect: Dialect) -> TypeEngine[Any]: ...
    def __getattr__(self, key: Any) -> Any: ...
    def process_literal_param(
        self, value: Optional[_T], dialect: Dialect
    ) -> str: ...
    def process_bind_param(
        self, value: Optional[_T], dialect: Dialect
    ) -> Any: ...
    def process_result_value(
        self, value: Any, dialect: Dialect
    ) -> Optional[_T]: ...
    def literal_processor(self, dialect: Dialect) -> _LiteralProcessor[_T]: ...
    def bind_processor(self, dialect: Dialect) -> _BindProcessor[_T]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> _ResultProcessor[_T]: ...
    def bind_expression(self, bindparam: Any) -> Any: ...
    def column_expression(self, column: Any) -> Any: ...
    def coerce_compared_value(self, op: Any, value: Any) -> Any: ...
    def copy(self: _TD, **kw: Any) -> _TD: ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...
    def compare_values(self, x: Any, y: Any) -> bool: ...
    @property
    def sort_key_function(self) -> Optional[compat._SortKeyFunction]: ...  # type: ignore[override]

class Variant(TypeDecorator[_T]):
    impl: TypeEngine[Any] = ...
    mapping: Mapping[str, TypeEngine[Any]] = ...
    def __init__(
        self, base: Any, mapping: Mapping[str, TypeEngine[Any]]
    ) -> None: ...
    def coerce_compared_value(
        self: _VT, operator: Any, value: Any
    ) -> Union[_VT, TypeEngine[Any]]: ...
    def load_dialect_impl(self, dialect: Dialect) -> TypeEngine[Any]: ...
    def with_variant(
        self, type_: Type[TypeEngine[_U]], dialect_name: str
    ) -> Variant[_U]: ...
    @property
    def comparator_factory(self) -> Type[Any]: ...  # type: ignore[override]

def to_instance(typeobj: Any, *arg: Any, **kw: Any) -> Any: ...
def adapt_type(typeobj: Any, colspecs: Any) -> Any: ...



from typing import Any

from .base import BIT as BIT
from .base import MySQLCompiler as MySQLCompiler
from .base import MySQLDialect as MySQLDialect
from .base import MySQLIdentifierPreparer as MySQLIdentifierPreparer
from ... import processors as processors
from ... import util as util

class MySQLCompiler_mysqlconnector(MySQLCompiler):
    def visit_mod_binary(self, binary: Any, operator: Any, **kw: Any): ...
    def post_process_text(self, text: Any): ...
    def escape_literal_column(self, text: Any): ...

class MySQLIdentifierPreparer_mysqlconnector(MySQLIdentifierPreparer): ...

class _myconnpyBIT(BIT):
    def result_processor(self, dialect: Any, coltype: Any) -> None: ...

class MySQLDialect_mysqlconnector(MySQLDialect):
    driver: str = ...
    supports_unicode_binds: bool = ...
    supports_sane_rowcount: bool = ...
    supports_sane_multi_rowcount: bool = ...
    supports_native_decimal: bool = ...
    default_paramstyle: str = ...
    statement_compiler: Any = ...
    preparer: Any = ...
    colspecs: Any = ...
    def __init__(self, *arg: Any, **kw: Any) -> None: ...
    @property
    def description_encoding(self): ...
    def supports_unicode_statements(self): ...
    @classmethod
    def dbapi(cls): ...
    def do_ping(self, dbapi_connection: Any): ...
    def create_connect_args(self, url: Any): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...

dialect = MySQLDialect_mysqlconnector



from typing import Any as _Any
from typing import Optional

from ... import types as sqltypes
from ... import util as util
from ...sql import coercions as coercions
from ...sql import expression as expression
from ...sql import operators as operators
from ...sql import roles as roles

def Any(other: _Any, arrexpr: _Any, operator: _Any = ...): ...
def All(other: _Any, arrexpr: _Any, operator: _Any = ...): ...

class array(expression.ClauseList, expression.ColumnElement):
    __visit_name__: str = ...
    stringify_dialect: str = ...
    type: _Any = ...
    def __init__(self, clauses: _Any, **kw: _Any) -> None: ...
    def self_group(self, against: Optional[_Any] = ...): ...

CONTAINS: _Any
CONTAINED_BY: _Any
OVERLAP: _Any

class ARRAY(sqltypes.ARRAY):
    class Comparator(sqltypes.ARRAY.Comparator):
        def contains(self, other: _Any, **kwargs: _Any): ...
        def contained_by(self, other: _Any): ...
        def overlap(self, other: _Any): ...
    comparator_factory: _Any = ...
    item_type: _Any = ...
    as_tuple: _Any = ...
    dimensions: _Any = ...
    zero_indexes: _Any = ...
    def __init__(
        self,
        item_type: _Any,
        as_tuple: bool = ...,
        dimensions: Optional[_Any] = ...,
        zero_indexes: bool = ...,
    ) -> None: ...
    @property
    def hashable(self): ...
    @property
    def python_type(self): ...
    def compare_values(self, x: _Any, y: _Any): ...
    def bind_expression(self, bindvalue: _Any): ...
    def bind_processor(self, dialect: _Any): ...
    def result_processor(self, dialect: _Any, coltype: _Any): ...



from builtins import _NotImplementedType
import sys
from types import MethodType
from typing import Any
from typing import Callable
from typing import Dict
from typing import Generic
from typing import Iterable
from typing import Iterator
from typing import List
from typing import Mapping
from typing import NoReturn
from typing import Optional
from typing import overload
from typing import Set
from typing import Tuple
from typing import TypeVar
from typing import Union

from _typeshed import SupportsKeysAndGetItem
from typing_extensions import Protocol

from .compat import _SortKeyFunction
from .compat import collections_abc as collections_abc
from .compat import threading

_I = TypeVar("_I", bound=immutabledict[Any, Any])
_PS = TypeVar("_PS", bound=Properties[str, Any])
_OD = TypeVar("_OD", bound=OrderedDict[Any, Any])
_OS = TypeVar("_OS", bound=OrderedSet[Any])
_IS = TypeVar("_IS", bound=IdentitySet[Any])
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
_NKT = TypeVar("_NKT")
_NVT = TypeVar("_NVT")
_T = TypeVar("_T")
_U = TypeVar("_U")

if sys.version_info >= (3, 0):
    _B = TypeVar("_B", bound=Union[str, bytes])
else:
    _B = TypeVar("_B", bound=Union[basestring, bytes])  # noqa: F821

EMPTY_SET: frozenset[None]

class ImmutableContainer:
    def __delitem__(self, key: Any) -> NoReturn: ...
    def __setitem__(self, key: Any, value: Any) -> NoReturn: ...
    def __setattr__(self, key: Any, value: Any) -> NoReturn: ...

class immutabledict(ImmutableContainer, Dict[_KT, _VT]):
    def clear(self) -> NoReturn: ...
    def pop(self, key: Any, default: Any = ...) -> NoReturn: ...
    def popitem(self) -> NoReturn: ...
    def setdefault(self, key: Any, default: Any = ...) -> NoReturn: ...
    def update(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def __reduce__(self) -> Tuple[Any, ...]: ...
    @overload
    def union(self: _I) -> _I: ...
    @overload
    def union(
        self, __d: Mapping[_NKT, _NVT]
    ) -> immutabledict[Union[_KT, _NKT], Union[_VT, _NVT]]: ...
    @overload
    def union(
        self, __d: Iterable[Tuple[_NKT, _NVT]]
    ) -> immutabledict[Union[_KT, _NKT], Union[_VT, _NVT]]: ...
    @overload
    def merge_with(self: _I) -> _I: ...
    @overload
    def merge_with(
        self, *dicts: Dict[_NKT, _NVT]
    ) -> immutabledict[Union[_KT, _NKT], Union[_VT, _NVT]]: ...

@overload
def coerce_to_immutabledict(d: None) -> immutabledict[Any, Any]: ...
@overload
def coerce_to_immutabledict(d: _I) -> _I: ...
@overload
def coerce_to_immutabledict(
    d: SupportsKeysAndGetItem[_KT, _VT]
) -> immutabledict[_KT, _VT]: ...
@overload
def coerce_to_immutabledict(
    d: Iterable[Tuple[_KT, _VT]]
) -> immutabledict[_KT, _VT]: ...

EMPTY_DICT: immutabledict[Any, Any]

class FacadeDict(ImmutableContainer, Dict[_KT, _VT]):
    def clear(self) -> NoReturn: ...
    def pop(self, key: Any, default: Any = ...) -> NoReturn: ...
    def popitem(self) -> NoReturn: ...
    def setdefault(self, key: Any, default: Any = ...) -> NoReturn: ...
    def update(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def copy(self) -> NoReturn: ...
    def __reduce__(self) -> Tuple[Any, ...]: ...

class Properties(Generic[_KT, _VT]):
    def __init__(self, data: Dict[_VT, _KT]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_VT]: ...
    def __dir__(self) -> Iterable[str]: ...
    def __add__(self, other: Any) -> List[str]: ...
    def __setitem__(self, k: _KT, v: _VT) -> None: ...
    def __getitem__(self, k: _KT) -> _VT: ...
    def __delitem__(self, v: _KT) -> None: ...
    def __setattr__(self: _PS, key: str, obj: Any) -> None: ...
    def __getattr__(self: _PS, key: str) -> _VT: ...
    def __contains__(self, key: Any) -> bool: ...
    def as_immutable(self) -> ImmutableProperties[_KT, _VT]: ...
    @overload
    def update(self, value: Mapping[_KT, _VT]) -> None: ...
    @overload
    def update(self, value: Iterable[Tuple[_KT, _VT]]) -> None: ...
    @overload
    def get(self, key: _KT) -> Optional[_VT]: ...
    @overload
    def get(self, key: _KT, default: Union[_VT, _T]) -> Union[_VT, _T]: ...
    def keys(self) -> List[_KT]: ...
    def values(self) -> List[_VT]: ...
    def items(self) -> List[Tuple[_KT, _VT]]: ...
    def has_key(self, key: _KT) -> bool: ...
    def clear(self) -> None: ...

class OrderedProperties(Properties[_KT, _VT]):
    def __init__(self) -> None: ...

class ImmutableProperties(ImmutableContainer, Properties[_KT, _VT]): ...

def sort_dictionary(
    d: OrderedDict[Any, Any], key: Optional[_SortKeyFunction] = ...
) -> None: ...

if sys.version_info >= (3, 7):
    # This is required because dict is not subscriptable before 3.9
    class OrderedDict(Dict[_KT, _VT]): ...

else:
    class OrderedDict(Dict[_KT, _VT]):
        def __reduce__(self) -> Tuple[Any, ...]: ...
        @overload
        def __init__(self, **kwargs: _VT) -> None: ...
        @overload
        def __init__(
            self, ____sequence: Mapping[_KT, _VT] = ..., **kwargs: Any
        ) -> None: ...
        @overload
        def __init__(
            self, ____sequence: Iterable[Tuple[_KT, _VT]] = ..., **kwargs: Any
        ) -> None: ...
        def clear(self) -> None: ...
        def copy(self: _OD) -> _OD: ...
        def __copy__(self: _OD) -> _OD: ...
        @overload
        def update(
            self, ____sequence: Mapping[_KT, _VT], **kwargs: _VT
        ) -> None: ...
        @overload
        def update(
            self, ____sequence: Iterable[Tuple[_KT, _VT]], **kwargs: _VT
        ) -> None: ...
        @overload
        def update(self, **kwargs: _VT) -> None: ...
        def setdefault(self, key: _KT, value: _VT = ...) -> _VT: ...
        def __iter__(self) -> Iterator[_KT]: ...
        def keys(self) -> List[_KT]: ...  # type: ignore[override]
        def values(self) -> List[_VT]: ...  # type: ignore[override]
        def items(self) -> List[Tuple[_KT, _VT]]: ...  # type: ignore[override]
        if sys.version_info < (3, 0):
            def itervalues(self) -> Iterator[_VT]: ...
            def iterkeys(self) -> Iterator[_KT]: ...
            def iteritems(self) -> Iterator[Tuple[_KT, _VT]]: ...
        def __setitem__(self, key: _KT, obj: _VT) -> None: ...
        def __delitem__(self, key: _KT) -> None: ...
        @overload
        def pop(self, key: _KT) -> _VT: ...
        @overload
        def pop(
            self, key: _KT, default: Union[_VT, _T] = ...
        ) -> Union[_VT, _T]: ...
        def popitem(self) -> Tuple[_KT, _VT]: ...

class OrderedSet(Set[_T]):
    def __init__(self, d: Optional[Iterable[_T]] = ...) -> None: ...
    def add(self, element: _T) -> None: ...
    def remove(self, element: _T) -> None: ...
    def insert(self, pos: int, element: _T) -> None: ...
    def discard(self, element: _T) -> None: ...
    def clear(self) -> None: ...
    @overload
    def __getitem__(self, key: int) -> _T: ...
    @overload
    def __getitem__(self, s: slice) -> List[_T]: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __add__(self: _OS, other: Iterable[_T]) -> _OS: ...
    def update(self: _OS, iterable: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def __ior__(self: _OS, iterable: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def union(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def __or__(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def intersection(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def __and__(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def symmetric_difference(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def __xor__(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def difference(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def __sub__(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def intersection_update(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def __iand__(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def symmetric_difference_update(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def __ixor__(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def difference_update(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]
    def __isub__(self: _OS, other: Iterable[_T]) -> _OS: ...  # type: ignore[override]

class IdentitySet(Generic[_T]):
    def __init__(self, iterable: Optional[Iterable[_T]] = ...) -> None: ...
    def add(self, value: _T) -> None: ...
    def __contains__(self, value: Any) -> bool: ...
    def remove(self, value: _T) -> None: ...
    def discard(self, value: _T) -> None: ...
    def pop(self) -> _T: ...
    def clear(self) -> None: ...
    def __cmp__(self, other: Any) -> NoReturn: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def issubset(self, iterable: Iterable[Any]) -> bool: ...
    def __le__(self, other: Any) -> Union[_NotImplementedType, bool]: ...
    def __lt__(self, other: Any) -> Union[_NotImplementedType, bool]: ...
    def issuperset(self, iterable: Iterable[Any]) -> bool: ...
    def __ge__(self, other: Any) -> Union[_NotImplementedType, bool]: ...
    def __gt__(self, other: Any) -> Union[_NotImplementedType, bool]: ...
    def union(self, iterable: Iterable[_U]) -> IdentitySet[Union[_T, _U]]: ...
    def __or__(
        self, other: Iterable[_U]
    ) -> Union[_NotImplementedType, IdentitySet[Union[_T, _U]]]: ...
    def update(self, iterable: Iterable[_T]) -> None: ...
    def __ior__(self: _IS, other: Iterable[_T]) -> Union[_NotImplementedType, _IS]: ...  # type: ignore[misc]
    def difference(self: _IS, iterable: Iterable[_T]) -> _IS: ...
    def __sub__(
        self: _IS, other: Iterable[_T]
    ) -> Union[_NotImplementedType, _IS]: ...
    def difference_update(self, iterable: Iterable[_T]) -> None: ...
    def __isub__(
        self: _IS, other: Iterable[_T]
    ) -> Union[_NotImplementedType, _IS]: ...
    def intersection(
        self, iterable: Iterable[_U]
    ) -> IdentitySet[Union[_T, _U]]: ...
    def __and__(
        self, iterable: Iterable[_U]
    ) -> Union[_NotImplementedType, IdentitySet[Union[_T, _U]]]: ...
    def intersection_update(self, iterable: Iterable[_T]) -> None: ...
    def __iand__(self: _IS, other: Iterable[_T]) -> Union[_NotImplementedType, _IS]: ...  # type: ignore[misc]
    def symmetric_difference(
        self, iterable: Iterable[_U]
    ) -> IdentitySet[Union[_T, _U]]: ...
    def __xor__(
        self, other: Iterable[_U]
    ) -> Union[_NotImplementedType, IdentitySet[Union[_T, _U]]]: ...
    def symmetric_difference_update(
        self: _IS, iterable: Iterable[_T]
    ) -> None: ...
    def __ixor__(self: _IS, other: Iterable[_T]) -> Union[_NotImplementedType, _IS]: ...  # type: ignore
    def copy(self: _IS) -> _IS: ...
    def __copy__(self: _IS) -> _IS: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __hash__(self) -> int: ...

class WeakSequence(Generic[_T]):
    def __init__(self, __elements: Iterable[_T] = ...) -> None: ...
    def append(self, item: _T) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __getitem__(self, index: int) -> _T: ...

class OrderedIdentitySet(IdentitySet[_T]):
    def __init__(self, iterable: Optional[Iterable[_T]] = ...) -> None: ...

class PopulateDict(Dict[_KT, _VT]):
    creator: Callable[[_KT], _VT] = ...
    def __init__(self, creator: Callable[[_KT], _VT]) -> None: ...
    def __missing__(self, key: _KT) -> _VT: ...

class WeakPopulateDict(Dict[_KT, _VT]):
    creator: Callable[[Any, _KT], _VT] = ...
    weakself: Any = ...
    def __init__(self, creator_method: MethodType) -> None: ...
    def __missing__(self, key: _KT) -> _VT: ...

# The following two are required because builtins are not subscriptable before 3.9
class column_set(Set[_T]): ...
class column_dict(Dict[_KT, _VT]): ...

ordered_column_set = OrderedSet

def unique_list(
    seq: Iterable[_T], hashfunc: Optional[Callable[[_T], int]] = ...
) -> List[_T]: ...

class _DataAppender(Protocol[_T]):
    def append(self, value: _T) -> None: ...
    def __iter__(self) -> Iterable[_T]: ...

class _DataAdder(Protocol[_T]):
    def add(self, value: _T) -> None: ...
    def __iter__(self) -> Iterable[_T]: ...

class UniqueAppender(Generic[_T]):
    data: Any = ...
    @overload
    def __init__(self, data: Any, via: str) -> None: ...
    @overload
    def __init__(
        self, data: Union[_DataAppender[_T], _DataAdder[_T]]
    ) -> None: ...
    def append(self, item: _T) -> None: ...
    def __iter__(self) -> Iterator[_T]: ...

def coerce_generator_arg(arg: Any) -> Any: ...
def to_list(x: Any, default: Optional[Any] = ...) -> List[Any]: ...
def has_intersection(set_: Set[Any], iterable: Iterable[Any]) -> bool: ...
def to_set(x: Any) -> Set[Any]: ...
def to_column_set(x: Any) -> Set[Any]: ...
def update_copy(
    d: Dict[_KT, _VT],
    _new: Optional[
        Union[Mapping[_NKT, _NVT], Iterable[Tuple[_NKT, _NVT]]]
    ] = ...,
    **kw: _VT,
) -> Dict[Union[_KT, _NKT], Union[_VT, _NVT]]: ...
def flatten_iterator(x: Iterable[Any]) -> Iterator[Any]: ...

class LRUCache(Dict[_KT, _VT]):
    capacity: int = ...
    threshold: float = ...
    size_alert: Callable[[LRUCache[_KT, _VT]], Any] = ...
    def __init__(
        self,
        capacity: int = ...,
        threshold: float = ...,
        size_alert: Optional[Callable[[LRUCache[_KT, _VT]], Any]] = ...,
    ) -> None: ...
    @overload
    def get(self, key: _KT) -> Optional[_VT]: ...
    @overload
    def get(self, key: _KT, default: _T) -> Union[_VT, _T]: ...
    def __getitem__(self, key: _KT) -> _VT: ...
    def values(self) -> List[_VT]: ...  # type: ignore[override]
    def setdefault(self, key: _KT, value: _VT) -> Union[_KT, _VT]: ...  # type: ignore[override]
    def __setitem__(self, key: _KT, value: _VT) -> None: ...
    @property
    def size_threshold(self) -> float: ...

class ScopedRegistry(Generic[_T]):
    createfunc: Callable[[], _T] = ...
    scopefunc: Callable[[], Any] = ...
    registry: Dict[Any, _T] = ...
    def __init__(
        self, createfunc: Callable[[], _T], scopefunc: Callable[[], Any]
    ) -> None: ...
    def __call__(self) -> _T: ...
    def has(self) -> bool: ...
    def set(self, obj: _T) -> None: ...
    def clear(self) -> None: ...

class ThreadLocalRegistry(ScopedRegistry[_T]):
    createfunc: Callable[[], _T] = ...
    registry: threading.local = ...  # type: ignore[assignment]
    def __init__(self, createfunc: Callable[[], _T]) -> None: ...
    def __call__(self) -> _T: ...
    def has(self) -> bool: ...
    def set(self, obj: _T) -> None: ...
    def clear(self) -> None: ...

def has_dupes(sequence: Iterable[Any], target: Any) -> bool: ...



# fmt: off
from sqlalchemy import inspect as inspect
from sqlalchemy import Integer as Integer
from ... import create_engine as create_engine
from ... import exc as exc
from ...schema import Column as Column
from ...schema import DropConstraint as DropConstraint
from ...schema import ForeignKeyConstraint as ForeignKeyConstraint
from ...schema import MetaData as MetaData
from ...schema import Table as Table
from ...testing.provision import create_db as create_db
from ...testing.provision import drop_all_schema_objects_pre_tables as drop_all_schema_objects_pre_tables
from ...testing.provision import drop_db as drop_db
from ...testing.provision import get_temp_table_name as get_temp_table_name
from ...testing.provision import log as log
from ...testing.provision import run_reap_dbs as run_reap_dbs
from ...testing.provision import temp_table_keyword_args as temp_table_keyword_args



from typing import Any
from typing import Callable
from typing import Optional
from typing import Type
from typing import TypeVar

from .langhelpers import inject_docstring_text as inject_docstring_text

SQLALCHEMY_WARN_20: bool

_C = TypeVar("_C", bound=Type[Any])
_F = TypeVar("_F", bound=Callable[..., Any])

def warn_deprecated(msg: Any, version: Any, stacklevel: int = ...) -> None: ...
def warn_deprecated_limited(
    msg: Any, args: Any, version: Any, stacklevel: int = ...
) -> None: ...
def warn_deprecated_20(msg: Any, stacklevel: int = ...) -> None: ...
def deprecated_cls(
    version: Any, message: Any, constructor: str = ...
) -> Callable[[_C], _C]: ...
def deprecated_20_cls(
    clsname: Any,
    alternative: Optional[Any] = ...,
    constructor: str = ...,
    becomes_legacy: bool = ...,
) -> Callable[[_C], _C]: ...
def deprecated(
    version: Any,
    message: Optional[Any] = ...,
    add_deprecation_to_docstring: bool = ...,
    warning: Optional[Any] = ...,
    enable_warnings: bool = ...,
) -> Callable[[_F], _F]: ...
def moved_20(message: Any, **kw: Any) -> Callable[[_F], _F]: ...
def deprecated_20(
    api_name: Any,
    alternative: Optional[Any] = ...,
    becomes_legacy: bool = ...,
    **kw: Any,
) -> Callable[[_F], _F]: ...
def deprecated_params(**specs: Any) -> Callable[[_F], _F]: ...



from typing import Any

from . import instrumentation as instrumentation
from . import interfaces as interfaces
from . import mapperlib as mapperlib
from .attributes import QueryableAttribute as QueryableAttribute
from .query import Query as Query
from .scoping import scoped_session as scoped_session
from .session import Session as Session
from .session import sessionmaker as sessionmaker
from .. import event as event
from .. import exc as exc
from .. import util as util
from ..util.compat import inspect_getfullargspec as inspect_getfullargspec

class InstrumentationEvents(event.Events):
    def class_instrument(self, cls: Any) -> None: ...
    def class_uninstrument(self, cls: Any) -> None: ...
    def attribute_instrument(self, cls: Any, key: Any, inst: Any) -> None: ...

class _InstrumentationEventsHold:
    class_: Any = ...
    def __init__(self, class_: Any) -> None: ...
    dispatch: Any = ...

class InstanceEvents(event.Events):
    def first_init(self, manager: Any, cls: Any) -> None: ...
    def init(self, target: Any, args: Any, kwargs: Any) -> None: ...
    def init_failure(self, target: Any, args: Any, kwargs: Any) -> None: ...
    def load(self, target: Any, context: Any) -> None: ...
    def refresh(self, target: Any, context: Any, attrs: Any) -> None: ...
    def refresh_flush(
        self, target: Any, flush_context: Any, attrs: Any
    ) -> None: ...
    def expire(self, target: Any, attrs: Any) -> None: ...
    def pickle(self, target: Any, state_dict: Any) -> None: ...
    def unpickle(self, target: Any, state_dict: Any) -> None: ...

class _EventsHold(event.RefCollection):
    class_: Any = ...
    def __init__(self, class_: Any) -> None: ...
    class HoldEvents: ...
    def remove(self, event_key: Any) -> None: ...
    @classmethod
    def populate(cls, class_: Any, subject: Any) -> None: ...

class _InstanceEventsHold(_EventsHold):
    all_holds: Any = ...
    def resolve(self, class_: Any): ...
    class HoldInstanceEvents(_EventsHold.HoldEvents, InstanceEvents): ...
    dispatch: Any = ...

class MapperEvents(event.Events):
    def instrument_class(self, mapper: Any, class_: Any) -> None: ...
    def before_mapper_configured(self, mapper: Any, class_: Any) -> None: ...
    def mapper_configured(self, mapper: Any, class_: Any) -> None: ...
    def before_configured(self) -> None: ...
    def after_configured(self) -> None: ...
    def before_insert(
        self, mapper: Any, connection: Any, target: Any
    ) -> None: ...
    def after_insert(
        self, mapper: Any, connection: Any, target: Any
    ) -> None: ...
    def before_update(
        self, mapper: Any, connection: Any, target: Any
    ) -> None: ...
    def after_update(
        self, mapper: Any, connection: Any, target: Any
    ) -> None: ...
    def before_delete(
        self, mapper: Any, connection: Any, target: Any
    ) -> None: ...
    def after_delete(
        self, mapper: Any, connection: Any, target: Any
    ) -> None: ...

class _MapperEventsHold(_EventsHold):
    all_holds: Any = ...
    def resolve(self, class_: Any): ...
    class HoldMapperEvents(_EventsHold.HoldEvents, MapperEvents): ...
    dispatch: Any = ...

class SessionEvents(event.Events):
    def do_orm_execute(self, orm_execute_state: Any) -> None: ...
    def after_transaction_create(
        self, session: Any, transaction: Any
    ) -> None: ...
    def after_transaction_end(
        self, session: Any, transaction: Any
    ) -> None: ...
    def before_commit(self, session: Any) -> None: ...
    def after_commit(self, session: Any) -> None: ...
    def after_rollback(self, session: Any) -> None: ...
    def after_soft_rollback(
        self, session: Any, previous_transaction: Any
    ) -> None: ...
    def before_flush(
        self, session: Any, flush_context: Any, instances: Any
    ) -> None: ...
    def after_flush(self, session: Any, flush_context: Any) -> None: ...
    def after_flush_postexec(
        self, session: Any, flush_context: Any
    ) -> None: ...
    def after_begin(
        self, session: Any, transaction: Any, connection: Any
    ) -> None: ...
    def before_attach(self, session: Any, instance: Any) -> None: ...
    def after_attach(self, session: Any, instance: Any) -> None: ...
    def after_bulk_update(self, update_context: Any) -> None: ...
    def after_bulk_delete(self, delete_context: Any) -> None: ...
    def transient_to_pending(self, session: Any, instance: Any) -> None: ...
    def pending_to_transient(self, session: Any, instance: Any) -> None: ...
    def persistent_to_transient(self, session: Any, instance: Any) -> None: ...
    def pending_to_persistent(self, session: Any, instance: Any) -> None: ...
    def detached_to_persistent(self, session: Any, instance: Any) -> None: ...
    def loaded_as_persistent(self, session: Any, instance: Any) -> None: ...
    def persistent_to_deleted(self, session: Any, instance: Any) -> None: ...
    def deleted_to_persistent(self, session: Any, instance: Any) -> None: ...
    def deleted_to_detached(self, session: Any, instance: Any) -> None: ...
    def persistent_to_detached(self, session: Any, instance: Any) -> None: ...

class AttributeEvents(event.Events):
    def append(self, target: Any, value: Any, initiator: Any) -> None: ...
    def bulk_replace(
        self, target: Any, values: Any, initiator: Any
    ) -> None: ...
    def remove(self, target: Any, value: Any, initiator: Any) -> None: ...
    def set(
        self, target: Any, value: Any, oldvalue: Any, initiator: Any
    ) -> None: ...
    def init_scalar(self, target: Any, value: Any, dict_: Any) -> None: ...
    def init_collection(
        self, target: Any, collection: Any, collection_adapter: Any
    ) -> None: ...
    def dispose_collection(
        self, target: Any, collection: Any, collection_adapter: Any
    ) -> None: ...
    def modified(self, target: Any, initiator: Any) -> None: ...

class QueryEvents(event.Events):
    def before_compile(self, query: Any) -> None: ...
    def before_compile_update(
        self, query: Any, update_context: Any
    ) -> None: ...
    def before_compile_delete(
        self, query: Any, delete_context: Any
    ) -> None: ...



from typing import Any
from typing import Optional

from . import base as oracle
from .base import OracleCompiler as OracleCompiler
from .base import OracleDialect as OracleDialect
from .base import OracleExecutionContext as OracleExecutionContext
from ... import exc as exc
from ... import processors as processors
from ... import types as sqltypes
from ... import util as util
from ...util import compat as compat

class _OracleInteger(sqltypes.Integer):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleNumeric(sqltypes.Numeric):
    is_number: bool = ...
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any) -> None: ...

class _OracleBinaryFloat(_OracleNumeric):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleBINARY_FLOAT(_OracleBinaryFloat, oracle.BINARY_FLOAT): ...
class _OracleBINARY_DOUBLE(_OracleBinaryFloat, oracle.BINARY_DOUBLE): ...

class _OracleNUMBER(_OracleNumeric):
    is_number: bool = ...

class _OracleDate(sqltypes.Date):
    def bind_processor(self, dialect: Any) -> None: ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class _OracleChar(sqltypes.CHAR):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleNChar(sqltypes.NCHAR):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleUnicodeStringNCHAR(oracle.NVARCHAR2):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleUnicodeStringCHAR(sqltypes.Unicode):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleUnicodeTextNCLOB(oracle.NCLOB):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleUnicodeTextCLOB(sqltypes.UnicodeText):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleText(sqltypes.Text):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleLong(oracle.LONG):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleString(sqltypes.String): ...

class _OracleEnum(sqltypes.Enum):
    def bind_processor(self, dialect: Any): ...

class _OracleBinary(sqltypes.LargeBinary):
    def get_dbapi_type(self, dbapi: Any): ...
    def bind_processor(self, dialect: Any) -> None: ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class _OracleInterval(oracle.INTERVAL):
    def get_dbapi_type(self, dbapi: Any): ...

class _OracleRaw(oracle.RAW): ...

class _OracleRowid(oracle.ROWID):
    def get_dbapi_type(self, dbapi: Any): ...

class OracleCompiler_cx_oracle(OracleCompiler):
    def bindparam_string(self, name: Any, **kw: Any): ...

class OracleExecutionContext_cx_oracle(OracleExecutionContext):
    out_parameters: Any = ...
    include_set_input_sizes: Any = ...
    def pre_exec(self) -> None: ...
    cursor_fetch_strategy: Any = ...
    def post_exec(self) -> None: ...
    def create_cursor(self): ...
    def get_out_parameter_values(self, out_param_names: Any): ...

class OracleDialect_cx_oracle(OracleDialect):
    execution_ctx_cls: Any = ...
    statement_compiler: Any = ...
    supports_sane_rowcount: bool = ...
    supports_sane_multi_rowcount: bool = ...
    supports_unicode_statements: bool = ...
    supports_unicode_binds: bool = ...
    use_setinputsizes: bool = ...
    driver: str = ...
    colspecs: Any = ...
    execute_sequence_format: Any = ...
    arraysize: Any = ...
    encoding_errors: Any = ...
    auto_convert_lobs: Any = ...
    coerce_to_unicode: Any = ...
    coerce_to_decimal: Any = ...
    cx_oracle_ver: Any = ...
    def __init__(
        self,
        auto_convert_lobs: bool = ...,
        coerce_to_unicode: bool = ...,
        coerce_to_decimal: bool = ...,
        arraysize: int = ...,
        encoding_errors: Optional[Any] = ...,
        threaded: Optional[Any] = ...,
        **kwargs: Any,
    ): ...
    @classmethod
    def dbapi(cls): ...
    def initialize(self, connection: Any) -> None: ...
    def get_isolation_level(self, connection: Any): ...
    def set_isolation_level(self, connection: Any, level: Any) -> None: ...
    def on_connect(self): ...
    def create_connect_args(self, url: Any): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...
    def create_xid(self): ...
    def do_executemany(
        self,
        cursor: Any,
        statement: Any,
        parameters: Any,
        context: Optional[Any] = ...,
    ) -> None: ...
    def do_begin_twophase(self, connection: Any, xid: Any) -> None: ...
    def do_prepare_twophase(self, connection: Any, xid: Any) -> None: ...
    def do_rollback_twophase(
        self,
        connection: Any,
        xid: Any,
        is_prepared: bool = ...,
        recover: bool = ...,
    ) -> None: ...
    def do_commit_twophase(
        self,
        connection: Any,
        xid: Any,
        is_prepared: bool = ...,
        recover: bool = ...,
    ) -> None: ...
    def do_set_input_sizes(
        self, cursor: Any, list_of_tuples: Any, context: Any
    ) -> None: ...
    def do_recover_twophase(self, connection: Any) -> None: ...

dialect = OracleDialect_cx_oracle



from typing import Any
from typing import Optional

from . import attributes as attributes
from . import interfaces as interfaces
from . import object_mapper as object_mapper
from . import object_session as object_session
from . import relationships as relationships
from . import strategies as strategies
from .query import Query as Query
from .. import exc as exc
from .. import log as log
from .. import sql as sql
from .. import util as util
from ..sql import selectable as selectable
from ..sql.base import Generative as Generative

class DynaLoader(strategies.AbstractRelationshipLoader):
    is_class_level: bool = ...
    def init_class_attribute(self, mapper: Any) -> None: ...

class DynamicAttributeImpl(attributes.AttributeImpl):
    uses_objects: bool = ...
    default_accepts_scalar_loader: bool = ...
    supports_population: bool = ...
    collection: bool = ...
    dynamic: bool = ...
    target_mapper: Any = ...
    order_by: Any = ...
    query_class: Any = ...
    def __init__(
        self,
        class_: Any,
        key: Any,
        typecallable: Any,
        dispatch: Any,
        target_mapper: Any,
        order_by: Any,
        **kw: Any,
    ) -> None: ...
    def get(self, state: Any, dict_: Any, passive: Any = ...): ...
    def get_collection(
        self,
        state: Any,
        dict_: Any,
        user_data: Optional[Any] = ...,
        passive: Any = ...,
    ): ...
    def fire_append_event(
        self,
        state: Any,
        dict_: Any,
        value: Any,
        initiator: Any,
        collection_history: Optional[Any] = ...,
    ) -> None: ...
    def fire_remove_event(
        self,
        state: Any,
        dict_: Any,
        value: Any,
        initiator: Any,
        collection_history: Optional[Any] = ...,
    ) -> None: ...
    def set(
        self,
        state: Any,
        dict_: Any,
        value: Any,
        initiator: Optional[Any] = ...,
        passive: Any = ...,
        check_old: Optional[Any] = ...,
        pop: bool = ...,
        _adapt: bool = ...,
    ) -> None: ...
    def delete(self, *args: Any, **kwargs: Any) -> None: ...
    def set_committed_value(
        self, state: Any, dict_: Any, value: Any
    ) -> None: ...
    def get_history(self, state: Any, dict_: Any, passive: Any = ...): ...
    def get_all_pending(self, state: Any, dict_: Any, passive: Any = ...): ...
    def append(
        self,
        state: Any,
        dict_: Any,
        value: Any,
        initiator: Any,
        passive: Any = ...,
    ) -> None: ...
    def remove(
        self,
        state: Any,
        dict_: Any,
        value: Any,
        initiator: Any,
        passive: Any = ...,
    ) -> None: ...
    def pop(
        self,
        state: Any,
        dict_: Any,
        value: Any,
        initiator: Any,
        passive: Any = ...,
    ) -> None: ...

class AppenderQuery(Generative):
    instance: Any = ...
    attr: Any = ...
    mapper: Any = ...
    def __init__(self, attr: Any, state: Any) -> None: ...
    def autoflush(self, setting: Any) -> None: ...
    @property
    def statement(self): ...
    def filter(self, *criteria: Any): ...
    def where(self, *criteria: Any) -> None: ...
    def order_by(self, *criteria: Any) -> None: ...
    def filter_by(self, **kwargs: Any) -> None: ...
    def join(self, target: Any, *props: Any, **kwargs: Any) -> None: ...
    def outerjoin(self, target: Any, *props: Any, **kwargs: Any) -> None: ...
    def scalar(self): ...
    def first(self): ...
    def one(self): ...
    def one_or_none(self): ...
    def all(self): ...
    def session(self): ...
    session: Any = ...
    def __iter__(self) -> Any: ...
    def __getitem__(self, index: Any): ...
    def limit(self, limit: Any) -> None: ...
    def offset(self, offset: Any) -> None: ...
    def slice(self, start: Any, stop: Any) -> None: ...
    def count(self): ...
    def extend(self, iterator: Any) -> None: ...
    def append(self, item: Any) -> None: ...
    def remove(self, item: Any) -> None: ...

class CollectionHistory:
    unchanged_items: Any = ...
    added_items: Any = ...
    deleted_items: Any = ...
    def __init__(
        self, attr: Any, state: Any, apply_to: Optional[Any] = ...
    ) -> None: ...
    @property
    def added_plus_unchanged(self): ...
    @property
    def all_items(self): ...
    def as_history(self): ...
    def indexed(self, index: Any): ...
    def add_added(self, value: Any) -> None: ...
    def add_removed(self, value: Any) -> None: ...



from typing import Any
from typing import Iterable
from typing import Iterator
from typing import List
from typing import Set
from typing import Tuple
from typing import TypeVar

_T = TypeVar("_T")

def sort_as_subsets(
    tuples: Iterable[Tuple[Any, _T]], allitems: Iterable[_T]
) -> Iterator[List[_T]]: ...
def sort(
    tuples: Iterable[Tuple[Any, _T]],
    allitems: Iterable[_T],
    deterministic_order: bool = ...,
) -> Iterator[_T]: ...
def find_cycles(
    tuples: Iterable[Tuple[Any, ...]], allitems: Iterable[Any]
) -> Set[Any]: ...



from typing import Any

from mypy.plugin import Callable as Callable
from mypy.plugin import FunctionContext as FunctionContext
from mypy.plugin import Optional as Optional
from mypy.plugin import Plugin
from mypy.plugin import Type as Type
from mypy.types import CallableType as CallableType

class CustomPlugin(Plugin):
    def get_function_hook(
        self, fullname: str
    ) -> Optional[Callable[[FunctionContext], Type]]: ...

def make_dec_thing(name: Any): ...

COLUMN_THINGY: int
RELATIONSHIP_THINGY: int

def plugin(version: str) -> Any: ...



from typing import Any
from typing import Optional

class _PlainColumnGetter:
    cols: Any = ...
    composite: Any = ...
    def __init__(self, cols: Any) -> None: ...
    def __reduce__(self): ...
    def __call__(self, value: Any): ...

class _SerializableColumnGetter:
    colkeys: Any = ...
    composite: Any = ...
    def __init__(self, colkeys: Any) -> None: ...
    def __reduce__(self): ...
    def __call__(self, value: Any): ...

class _SerializableColumnGetterV2(_PlainColumnGetter):
    colkeys: Any = ...
    composite: Any = ...
    def __init__(self, colkeys: Any) -> None: ...
    def __reduce__(self): ...

def column_mapped_collection(mapping_spec: Any): ...

class _SerializableAttrGetter:
    name: Any = ...
    getter: Any = ...
    def __init__(self, name: Any) -> None: ...
    def __call__(self, target: Any): ...
    def __reduce__(self): ...

def attribute_mapped_collection(attr_name: Any): ...
def mapped_collection(keyfunc: Any): ...

class collection:
    @staticmethod
    def appender(fn: Any): ...
    @staticmethod
    def remover(fn: Any): ...
    @staticmethod
    def iterator(fn: Any): ...
    @staticmethod
    def internally_instrumented(fn: Any): ...
    @staticmethod
    def converter(fn: Any): ...
    @staticmethod
    def adds(arg: Any): ...
    @staticmethod
    def replaces(arg: Any): ...
    @staticmethod
    def removes(arg: Any): ...
    @staticmethod
    def removes_return(): ...

collection_adapter: Any

class CollectionAdapter:
    attr: Any = ...
    owner_state: Any = ...
    invalidated: bool = ...
    empty: bool = ...
    def __init__(self, attr: Any, owner_state: Any, data: Any) -> None: ...
    @property
    def data(self): ...
    def bulk_appender(self): ...
    def append_with_event(
        self, item: Any, initiator: Optional[Any] = ...
    ) -> None: ...
    def append_without_event(self, item: Any) -> None: ...
    def append_multiple_without_event(self, items: Any) -> None: ...
    def bulk_remover(self): ...
    def remove_with_event(
        self, item: Any, initiator: Optional[Any] = ...
    ) -> None: ...
    def remove_without_event(self, item: Any) -> None: ...
    def clear_with_event(self, initiator: Optional[Any] = ...) -> None: ...
    def clear_without_event(self) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self): ...
    def __bool__(self): ...
    __nonzero__: Any = ...
    def fire_append_event(self, item: Any, initiator: Optional[Any] = ...): ...
    def fire_remove_event(
        self, item: Any, initiator: Optional[Any] = ...
    ) -> None: ...
    def fire_pre_remove_event(
        self, initiator: Optional[Any] = ...
    ) -> None: ...

class InstrumentedList(list): ...
class InstrumentedSet(set): ...
class InstrumentedDict(dict): ...

class MappedCollection(dict):
    keyfunc: Any = ...
    def __init__(self, keyfunc: Any) -> None: ...
    def set(self, value: Any, _sa_initiator: Optional[Any] = ...) -> None: ...
    def remove(
        self, value: Any, _sa_initiator: Optional[Any] = ...
    ) -> None: ...



from typing import Any
from typing import Optional
from typing import TypeVar

from . import attributes as attributes
from . import util as orm_util
from .interfaces import MapperProperty as MapperProperty
from .interfaces import PropComparator as PropComparator
from .. import event as event
from .. import schema as schema
from .. import sql as sql
from .. import util as util
from ..sql import expression as expression

_T = TypeVar("_T")

class DescriptorProperty(MapperProperty[_T]):
    doc: Any = ...
    uses_objects: bool = ...
    key: Any = ...
    descriptor: Any = ...
    def instrument_class(self, mapper: Any): ...

class CompositeProperty(DescriptorProperty[_T]):
    attrs: Any = ...
    composite_class: Any = ...
    active_history: Any = ...
    deferred: Any = ...
    group: Any = ...
    comparator_factory: Any = ...
    info: Any = ...
    def __init__(self, class_: Any, *attrs: Any, **kwargs: Any) -> None: ...
    def instrument_class(self, mapper: Any) -> None: ...
    def do_init(self) -> None: ...
    def props(self): ...
    @property
    def columns(self): ...
    def get_history(self, state: Any, dict_: Any, passive: Any = ...): ...
    class CompositeBundle(orm_util.Bundle):
        property: Any = ...
        def __init__(self, property_: Any, expr: Any) -> None: ...
        def create_row_processor(
            self, query: Any, procs: Any, labels: Any
        ): ...
    class Comparator(PropComparator):
        __hash__: Any = ...
        def clauses(self): ...
        def __clause_element__(self): ...
        def expression(self): ...
        def __eq__(self, other: Any) -> Any: ...
        def __ne__(self, other: Any) -> Any: ...

class ConcreteInheritedProperty(DescriptorProperty[_T]):
    descriptor: Any = ...
    def __init__(self): ...

class SynonymProperty(DescriptorProperty[_T]):
    name: Any = ...
    map_column: Any = ...
    descriptor: Any = ...
    comparator_factory: Any = ...
    doc: Any = ...
    info: Any = ...
    def __init__(
        self,
        name: Any,
        map_column: Optional[Any] = ...,
        descriptor: Optional[Any] = ...,
        comparator_factory: Optional[Any] = ...,
        doc: Optional[Any] = ...,
        info: Optional[Any] = ...,
    ) -> None: ...
    @property
    def uses_objects(self): ...
    def get_history(self, *arg: Any, **kw: Any): ...
    parent: Any = ...
    def set_parent(self, parent: Any, init: Any) -> None: ...



from typing import Any
from typing import Dict
from typing import List
from typing import Mapping
from typing import NamedTuple
from typing import Optional
from typing import Sequence
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union

from .interfaces import Dialect
from .. import util

_TURL = TypeVar("_TURL", bound=URL)

class URL(NamedTuple):
    drivername: str = ...
    username: Optional[str] = ...
    password: Optional[Union[str, object]] = ...
    host: Optional[str] = ...
    port: Optional[int] = ...
    database: Optional[str] = ...
    query: Mapping[str, Union[str, Sequence[str]]] = ...
    def __new__(self: Type[_TURL], *arg: Any, **kw: Any) -> _TURL: ...  # type: ignore[misc]
    @classmethod
    def create(
        cls: Type[_TURL],
        drivername: str,
        username: Optional[str] = ...,
        password: Optional[Union[str, object]] = ...,
        host: Optional[str] = ...,
        port: Optional[int] = ...,
        database: Optional[str] = ...,
        query: Mapping[str, Union[str, Sequence[str]]] = ...,
    ) -> _TURL: ...
    def set(
        self: _TURL,
        drivername: Optional[str] = ...,
        username: Optional[str] = ...,
        password: Optional[Union[str, object]] = ...,
        host: Optional[str] = ...,
        port: Optional[int] = ...,
        database: Optional[str] = ...,
        query: Optional[Mapping[str, Union[str, Sequence[str]]]] = ...,
    ) -> _TURL: ...
    def update_query_string(
        self: _TURL, query_string: str, append: bool = ...
    ) -> _TURL: ...
    def update_query_pairs(
        self: _TURL,
        key_value_pairs: Sequence[Tuple[str, str]],
        append: bool = ...,
    ) -> _TURL: ...
    def update_query_dict(
        self: _TURL,
        query_parameters: Mapping[str, Union[str, Sequence[str]]],
        append: bool = ...,
    ) -> _TURL: ...
    def difference_update_query(
        self: _TURL, names: Sequence[str]
    ) -> _TURL: ...
    @util.memoized_property
    def normalized_query(self) -> util.immutabledict[str, Any]: ...
    def __to_string__(self, hide_password: bool = ...) -> str: ...
    def render_as_string(self, hide_password: bool = ...) -> str: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def get_backend_name(self) -> str: ...
    def get_driver_name(self) -> str: ...
    def get_dialect(self) -> Type[Dialect]: ...
    def translate_connect_args(
        self, names: List[str] = ..., **kw: Any
    ) -> Dict[str, Any]: ...

def make_url(name_or_url: Union[str, URL]) -> URL: ...



from typing import Any

from .base import Pool as Pool
from .. import event as event
from ..engine.base import Engine as Engine

class PoolEvents(event.Events):
    def connect(
        self, dbapi_connection: Any, connection_record: Any
    ) -> None: ...
    def first_connect(
        self, dbapi_connection: Any, connection_record: Any
    ) -> None: ...
    def checkout(
        self,
        dbapi_connection: Any,
        connection_record: Any,
        connection_proxy: Any,
    ) -> None: ...
    def checkin(
        self, dbapi_connection: Any, connection_record: Any
    ) -> None: ...
    def reset(self, dbapi_connection: Any, connection_record: Any) -> None: ...
    def invalidate(
        self, dbapi_connection: Any, connection_record: Any, exception: Any
    ) -> None: ...
    def soft_invalidate(
        self, dbapi_connection: Any, connection_record: Any, exception: Any
    ) -> None: ...
    def close(self, dbapi_connection: Any, connection_record: Any) -> None: ...
    def detach(
        self, dbapi_connection: Any, connection_record: Any
    ) -> None: ...
    def close_detached(self, dbapi_connection: Any) -> None: ...



from typing import Any

from .base import BIGINT as BIGINT
from .base import BINARY as BINARY
from .base import BIT as BIT
from .base import BLOB as BLOB
from .base import BOOLEAN as BOOLEAN
from .base import CHAR as CHAR
from .base import DATE as DATE
from .base import DATETIME as DATETIME
from .base import DECIMAL as DECIMAL
from .base import DOUBLE as DOUBLE
from .base import ENUM as ENUM
from .base import FLOAT as FLOAT
from .base import INTEGER as INTEGER
from .base import JSON as JSON
from .base import LONGBLOB as LONGBLOB
from .base import LONGTEXT as LONGTEXT
from .base import MEDIUMBLOB as MEDIUMBLOB
from .base import MEDIUMINT as MEDIUMINT
from .base import MEDIUMTEXT as MEDIUMTEXT
from .base import NCHAR as NCHAR
from .base import NUMERIC as NUMERIC
from .base import NVARCHAR as NVARCHAR
from .base import REAL as REAL
from .base import SET as SET
from .base import SMALLINT as SMALLINT
from .base import TEXT as TEXT
from .base import TIME as TIME
from .base import TIMESTAMP as TIMESTAMP
from .base import TINYBLOB as TINYBLOB
from .base import TINYINT as TINYINT
from .base import TINYTEXT as TINYTEXT
from .base import VARBINARY as VARBINARY
from .base import VARCHAR as VARCHAR
from .base import YEAR as YEAR
from .dml import Insert as Insert
from .dml import insert as insert

dialect: Any



import decimal
from typing import Any
from typing import Generic
from typing import overload
from typing import Type
from typing import TypeVar
from typing import Union

_T = TypeVar("_T")

class registry:
    def __init__(self) -> None: ...
    def mapped(self, cls: type) -> type: ...

class TypeEngine(Generic[_T]):
    @property
    def python_type(self) -> Type[_T]: ...

class Integer(TypeEngine[int]):
    @property
    def python_type(self) -> Type[int]: ...

class String(TypeEngine[str]):
    @property
    def python_type(self) -> Type[str]: ...

class Numeric(TypeEngine[Union[decimal.Decimal, float]]):
    @property
    def python_type(self) -> Type[Union[decimal.Decimal, float]]: ...

class Float(Numeric):
    @property
    def python_type(self) -> Type[Union[decimal.Decimal, float]]: ...

class Column:
    type: Any = ...
    def __init__(self, type_: TypeEngine[_T]) -> None: ...

class relationship:
    type: Any = ...
    def __init__(self, type_: TypeEngine[_T]) -> None: ...

class Mapped:
    @overload
    def __get__(self, instance: None, owner: Any) -> Mapped[_T]: ...
    @overload
    def __get__(self, instance: object, owner: Any) -> Union[_T, None]: ...
    # TODO: this is a hack to work with the plugin for now, needs adjustment
    @classmethod
    def _empty_constructor(cls, _T) -> "Mapped[_T]": ...
    @property
    def property(self) -> object: ...
    # so that it can work with the "constructor", or none
    # def __init__(self, type: _T) -> None: ...



from typing import Any
from typing import Optional

from ... import Computed as Computed
from ... import exc as exc
from ... import sql as sql
from ... import util as util
from ...engine import default as default
from ...engine import reflection as reflection
from ...sql import compiler as compiler
from ...sql import expression as expression
from ...sql import sqltypes as sqltypes
from ...sql import visitors as visitors
from ...types import BLOB as BLOB
from ...types import CHAR as CHAR
from ...types import CLOB as CLOB
from ...types import FLOAT as FLOAT
from ...types import INTEGER as INTEGER
from ...types import Integer as Integer
from ...types import NCHAR as NCHAR
from ...types import NVARCHAR as NVARCHAR
from ...types import TIMESTAMP as TIMESTAMP
from ...types import VARCHAR as VARCHAR
from ...util import compat as compat

RESERVED_WORDS: Any
NO_ARG_FNS: Any

class RAW(sqltypes._Binary):
    __visit_name__: str = ...

OracleRaw = RAW

class NCLOB(sqltypes.Text):
    __visit_name__: str = ...

class VARCHAR2(VARCHAR):
    __visit_name__: str = ...

NVARCHAR2 = NVARCHAR

class NUMBER(sqltypes.Numeric, sqltypes.Integer):
    __visit_name__: str = ...
    def __init__(
        self,
        precision: Optional[Any] = ...,
        scale: Optional[Any] = ...,
        asdecimal: Optional[Any] = ...,
    ) -> None: ...
    def adapt(self, impltype: Any): ...

class DOUBLE_PRECISION(sqltypes.Float):
    __visit_name__: str = ...

class BINARY_DOUBLE(sqltypes.Float):
    __visit_name__: str = ...

class BINARY_FLOAT(sqltypes.Float):
    __visit_name__: str = ...

class BFILE(sqltypes.LargeBinary):
    __visit_name__: str = ...

class LONG(sqltypes.Text):
    __visit_name__: str = ...

class DATE(sqltypes.DateTime):
    __visit_name__: str = ...

class INTERVAL(sqltypes.NativeForEmulated, sqltypes._AbstractInterval):
    __visit_name__: str = ...
    day_precision: Any = ...
    second_precision: Any = ...
    def __init__(
        self,
        day_precision: Optional[Any] = ...,
        second_precision: Optional[Any] = ...,
    ) -> None: ...
    def as_generic(self, allow_nulltype: bool = ...): ...

class ROWID(sqltypes.TypeEngine):
    __visit_name__: str = ...

class _OracleBoolean(sqltypes.Boolean):
    def get_dbapi_type(self, dbapi: Any): ...

colspecs: Any
ischema_names: Any

class OracleTypeCompiler(compiler.GenericTypeCompiler):
    def visit_datetime(self, type_: Any, **kw: Any): ...
    def visit_float(self, type_: Any, **kw: Any): ...
    def visit_unicode(self, type_: Any, **kw: Any): ...
    def visit_INTERVAL(self, type_: Any, **kw: Any): ...
    def visit_LONG(self, type_: Any, **kw: Any): ...
    def visit_TIMESTAMP(self, type_: Any, **kw: Any): ...
    def visit_DOUBLE_PRECISION(self, type_: Any, **kw: Any): ...
    def visit_BINARY_DOUBLE(self, type_: Any, **kw: Any): ...
    def visit_BINARY_FLOAT(self, type_: Any, **kw: Any): ...
    def visit_FLOAT(self, type_: Any, **kw: Any): ...
    def visit_NUMBER(self, type_: Any, **kw: Any): ...
    def visit_string(self, type_: Any, **kw: Any): ...
    def visit_VARCHAR2(self, type_: Any, **kw: Any): ...
    def visit_NVARCHAR2(self, type_: Any, **kw: Any): ...
    visit_NVARCHAR: Any = ...
    def visit_VARCHAR(self, type_: Any, **kw: Any): ...
    def visit_text(self, type_: Any, **kw: Any): ...
    def visit_unicode_text(self, type_: Any, **kw: Any): ...
    def visit_large_binary(self, type_: Any, **kw: Any): ...
    def visit_big_integer(self, type_: Any, **kw: Any): ...
    def visit_boolean(self, type_: Any, **kw: Any): ...
    def visit_RAW(self, type_: Any, **kw: Any): ...
    def visit_ROWID(self, type_: Any, **kw: Any): ...

class OracleCompiler(compiler.SQLCompiler):
    compound_keywords: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def visit_mod_binary(self, binary: Any, operator: Any, **kw: Any): ...
    def visit_now_func(self, fn: Any, **kw: Any): ...
    def visit_char_length_func(self, fn: Any, **kw: Any): ...
    def visit_match_op_binary(self, binary: Any, operator: Any, **kw: Any): ...
    def visit_true(self, expr: Any, **kw: Any): ...
    def visit_false(self, expr: Any, **kw: Any): ...
    def get_cte_preamble(self, recursive: Any): ...
    def get_select_hint_text(self, byfroms: Any): ...
    def function_argspec(self, fn: Any, **kw: Any): ...
    def visit_function(self, func: Any, **kw: Any): ...
    def visit_table_valued_column(self, element: Any, **kw: Any): ...
    def default_from(self): ...
    def visit_join(
        self, join: Any, from_linter: Optional[Any] = ..., **kwargs: Any
    ): ...
    def visit_outer_join_column(self, vc: Any, **kw: Any): ...
    def visit_sequence(self, seq: Any, **kw: Any): ...
    def get_render_as_alias_suffix(self, alias_name_text: Any): ...
    has_out_parameters: bool = ...
    def returning_clause(self, stmt: Any, returning_cols: Any): ...
    def translate_select_structure(self, select_stmt: Any, **kwargs: Any): ...
    def limit_clause(self, select: Any, **kw: Any): ...
    def visit_empty_set_expr(self, type_: Any): ...
    def for_update_clause(self, select: Any, **kw: Any): ...
    def visit_is_distinct_from_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_is_not_distinct_from_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_regexp_match_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_not_regexp_match_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_regexp_replace_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...

class OracleDDLCompiler(compiler.DDLCompiler):
    def define_constraint_cascades(self, constraint: Any): ...
    def visit_drop_table_comment(self, drop: Any): ...
    def visit_create_index(self, create: Any): ...
    def post_create_table(self, table: Any): ...
    def get_identity_options(self, identity_options: Any): ...
    def visit_computed_column(self, generated: Any): ...
    def visit_identity_column(self, identity: Any, **kw: Any): ...

class OracleIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any = ...
    illegal_initial_characters: Any = ...
    def format_savepoint(self, savepoint: Any): ...

class OracleExecutionContext(default.DefaultExecutionContext):
    def fire_sequence(self, seq: Any, type_: Any): ...

class OracleDialect(default.DefaultDialect):
    name: str = ...
    supports_alter: bool = ...
    supports_unicode_statements: bool = ...
    supports_unicode_binds: bool = ...
    max_identifier_length: int = ...
    supports_simple_order_by_label: bool = ...
    cte_follows_insert: bool = ...
    supports_sequences: bool = ...
    sequences_optional: bool = ...
    postfetch_lastrowid: bool = ...
    default_paramstyle: str = ...
    colspecs: Any = ...
    ischema_names: Any = ...
    requires_name_normalize: bool = ...
    supports_comments: bool = ...
    supports_default_values: bool = ...
    supports_empty_insert: bool = ...
    statement_compiler: Any = ...
    ddl_compiler: Any = ...
    type_compiler: Any = ...
    preparer: Any = ...
    execution_ctx_cls: Any = ...
    reflection_options: Any = ...
    construct_arguments: Any = ...
    use_ansi: Any = ...
    optimize_limits: Any = ...
    exclude_tablespaces: Any = ...
    def __init__(
        self,
        use_ansi: bool = ...,
        optimize_limits: bool = ...,
        use_binds_for_limits: Optional[Any] = ...,
        use_nchar_for_unicode: bool = ...,
        exclude_tablespaces: Any = ...,
        **kwargs: Any,
    ) -> None: ...
    implicit_returning: Any = ...
    def initialize(self, connection: Any) -> None: ...
    def do_release_savepoint(self, connection: Any, name: Any) -> None: ...
    def get_isolation_level(self, connection: Any) -> None: ...
    def get_default_isolation_level(self, dbapi_conn: Any): ...
    def set_isolation_level(self, connection: Any, level: Any) -> None: ...
    def has_table(
        self, connection: Any, table_name: Any, schema: Optional[Any] = ...
    ): ...
    def has_sequence(
        self, connection: Any, sequence_name: Any, schema: Optional[Any] = ...
    ): ...
    def get_schema_names(self, connection: Any, **kw: Any): ...
    def get_table_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_temp_table_names(self, connection: Any, **kw: Any): ...
    def get_view_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_sequence_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_table_options(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_columns(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_table_comment(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        resolve_synonyms: bool = ...,
        dblink: str = ...,
        **kw: Any,
    ): ...
    def get_indexes(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        resolve_synonyms: bool = ...,
        dblink: str = ...,
        **kw: Any,
    ): ...
    def get_pk_constraint(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_foreign_keys(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_unique_constraints(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_view_definition(
        self,
        connection: Any,
        view_name: Any,
        schema: Optional[Any] = ...,
        resolve_synonyms: bool = ...,
        dblink: str = ...,
        **kw: Any,
    ): ...
    def get_check_constraints(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        include_all: bool = ...,
        **kw: Any,
    ): ...

class _OuterJoinColumn(sql.ClauseElement):
    __visit_name__: str = ...
    column: Any = ...
    def __init__(self, column: Any) -> None: ...



from typing import Any
from typing import Optional

from . import Connector as Connector
from .. import util as util

class PyODBCConnector(Connector):
    driver: str = ...
    supports_sane_rowcount_returning: bool = ...
    supports_sane_multi_rowcount: bool = ...
    supports_unicode_statements: bool = ...
    supports_unicode_binds: bool = ...
    supports_native_decimal: bool = ...
    default_paramstyle: str = ...
    use_setinputsizes: bool = ...
    pyodbc_driver_name: Any = ...
    def __init__(
        self, supports_unicode_binds: Optional[Any] = ..., **kw: Any
    ) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(self, url: Any): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...
    def do_set_input_sizes(
        self, cursor: Any, list_of_tuples: Any, context: Any
    ) -> None: ...
    def set_isolation_level(self, connection: Any, level: Any) -> None: ...



from typing import Any
from typing import Callable
from typing import Dict
from typing import Iterable
from typing import List
from typing import Optional
from typing import Sequence
from typing import Set
from typing import Tuple
from typing import TypeVar
from typing import Union

from typing_extensions import Protocol

from . import roles
from .base import Executable
from .base import SchemaVisitor
from .elements import ClauseElement
from .schema import ForeignKey
from .schema import ForeignKeyConstraint
from .schema import Index
from .schema import SchemaItem
from .schema import Table
from ..engine import Connection
from ..engine import Dialect
from ..engine import Engine

_DDLE = TypeVar("_DDLE", bound=DDLElement)

_Bind = Union[Engine, Connection]

class _DDLElementCallback(Protocol):
    def __call__(
        self,
        __ddl: DDLElement,
        __target: Optional[SchemaItem],
        __bind: _Bind,
        *,
        state: Optional[Any],
        **kw: Any,
    ) -> bool: ...

class _DDLCompiles(ClauseElement): ...

class DDLElement(roles.DDLRole, Executable, _DDLCompiles):
    target: Optional[SchemaItem] = ...
    on: Any = ...
    dialect: Union[str, Tuple[str, ...], List[str], Set[str], None] = ...
    callable_: Optional[_DDLElementCallback] = ...
    state: Optional[Any] = ...
    bind: Optional[_Bind] = ...
    def execute(  # type: ignore[override]
        self,
        bind: Optional[_Bind] = ...,
        target: Optional[SchemaItem] = ...,
    ) -> Any: ...
    def against(self: _DDLE, target: Optional[SchemaItem]) -> _DDLE: ...
    def execute_if(
        self: _DDLE,
        dialect: Optional[
            Union[str, Tuple[str, ...], List[str], Set[str]]
        ] = ...,
        callable_: Optional[_DDLElementCallback] = ...,
        state: Optional[Any] = ...,
    ) -> _DDLE: ...
    def __call__(
        self, target: SchemaItem, bind: _Bind, **kw: Any
    ) -> Optional[Any]: ...

class DDL(DDLElement):
    __visit_name__: str = ...
    statement: str = ...
    context: Any = ...
    def __init__(
        self,
        statement: str,
        context: Optional[Dict[str, Any]] = ...,
        bind: Optional[_Bind] = ...,
    ) -> None: ...

class _CreateDropBase(DDLElement):
    element: Any = ...
    if_exists: bool = ...
    if_not_exists: bool = ...
    def __init__(
        self,
        element: Any,
        bind: Optional[_Bind] = ...,
        if_exists: bool = ...,
        if_not_exists: bool = ...,
        _legacy_bind: Optional[Any] = ...,
    ) -> None: ...
    @property
    def stringify_dialect(self) -> str: ...  # type: ignore[override]

class CreateSchema(_CreateDropBase):
    __visit_name__: str = ...
    quote: Any = ...
    def __init__(
        self, name: str, quote: Optional[Any] = ..., **kw: Any
    ) -> None: ...

class DropSchema(_CreateDropBase):
    __visit_name__: str = ...
    quote: Any = ...
    cascade: Any = ...
    def __init__(
        self,
        name: str,
        quote: Optional[Any] = ...,
        cascade: bool = ...,
        **kw: Any,
    ) -> None: ...

class CreateTable(_CreateDropBase):
    __visit_name__: str = ...
    columns: Any = ...
    include_foreign_key_constraints: Any = ...
    def __init__(
        self,
        element: Table,
        bind: Optional[_Bind] = ...,
        include_foreign_key_constraints: Optional[
            Sequence[ForeignKeyConstraint]
        ] = ...,
        if_not_exists: bool = ...,
    ) -> None: ...

class _DropView(_CreateDropBase):
    __visit_name__: str = ...

class CreateColumn(_DDLCompiles):
    __visit_name__: str = ...
    element: Any = ...
    def __init__(self, element: Any) -> None: ...

class DropTable(_CreateDropBase):
    __visit_name__: str = ...
    def __init__(
        self,
        element: Table,
        bind: Optional[_Bind] = ...,
        if_exists: bool = ...,
    ) -> None: ...

class CreateSequence(_CreateDropBase):
    __visit_name__: str = ...

class DropSequence(_CreateDropBase):
    __visit_name__: str = ...

class CreateIndex(_CreateDropBase):
    __visit_name__: str = ...
    def __init__(
        self,
        element: Index,
        bind: Optional[_Bind] = ...,
        if_not_exists: bool = ...,
    ) -> None: ...

class DropIndex(_CreateDropBase):
    __visit_name__: str = ...
    def __init__(
        self,
        element: Index,
        bind: Optional[_Bind] = ...,
        if_exists: bool = ...,
    ) -> None: ...

class AddConstraint(_CreateDropBase):
    __visit_name__: str = ...
    def __init__(self, element: Any, *args: Any, **kw: Any) -> None: ...

class DropConstraint(_CreateDropBase):
    __visit_name__: str = ...
    cascade: Any = ...
    def __init__(
        self, element: Any, cascade: bool = ..., **kw: Any
    ) -> None: ...

class SetTableComment(_CreateDropBase):
    __visit_name__: str = ...

class DropTableComment(_CreateDropBase):
    __visit_name__: str = ...

class SetColumnComment(_CreateDropBase):
    __visit_name__: str = ...

class DropColumnComment(_CreateDropBase):
    __visit_name__: str = ...

class DDLBase(SchemaVisitor):
    connection: Any = ...
    def __init__(self, connection: Any) -> None: ...

class SchemaGenerator(DDLBase):
    checkfirst: Any = ...
    tables: Any = ...
    preparer: Any = ...
    dialect: Dialect = ...
    memo: Any = ...
    def __init__(
        self,
        dialect: Dialect,
        connection: Any,
        checkfirst: bool = ...,
        tables: Optional[Any] = ...,
        **kwargs: Any,
    ) -> None: ...
    def visit_metadata(self, metadata: Any) -> None: ...
    def visit_table(
        self,
        table: Any,
        create_ok: bool = ...,
        include_foreign_key_constraints: Optional[Any] = ...,
        _is_metadata_operation: bool = ...,
    ) -> None: ...
    def visit_foreign_key_constraint(self, constraint: Any) -> None: ...
    def visit_sequence(self, sequence: Any, create_ok: bool = ...) -> None: ...
    def visit_index(self, index: Any, create_ok: bool = ...) -> None: ...

class SchemaDropper(DDLBase):
    checkfirst: Any = ...
    tables: Any = ...
    preparer: Any = ...
    dialect: Dialect = ...
    memo: Any = ...
    def __init__(
        self,
        dialect: Dialect,
        connection: Any,
        checkfirst: bool = ...,
        tables: Optional[Any] = ...,
        **kwargs: Any,
    ) -> None: ...
    def visit_metadata(self, metadata: Any) -> None: ...
    def visit_index(self, index: Any, drop_ok: bool = ...) -> None: ...
    def visit_table(
        self,
        table: Any,
        drop_ok: bool = ...,
        _is_metadata_operation: bool = ...,
    ) -> None: ...
    def visit_foreign_key_constraint(self, constraint: Any) -> None: ...
    def visit_sequence(self, sequence: Any, drop_ok: bool = ...) -> None: ...

def sort_tables(
    tables: Iterable[Table],
    skip_fn: Optional[Callable[[ForeignKey], bool]] = ...,
    extra_dependencies: Optional[Iterable[Tuple[Table, Table]]] = ...,
) -> List[Table]: ...
def sort_tables_and_constraints(
    tables: Iterable[Table],
    filter_fn: Optional[
        Callable[[ForeignKeyConstraint], Optional[bool]]
    ] = ...,
    extra_dependencies: Optional[Iterable[Tuple[Table, Table]]] = ...,
    _warn_for_cycles: bool = ...,
) -> List[Tuple[Optional[Table], List[ForeignKeyConstraint]]]: ...



from typing import Any
from typing import Optional

from .base import MySQLCompiler as MySQLCompiler
from .base import MySQLDialect as MySQLDialect
from .base import MySQLExecutionContext as MySQLExecutionContext
from .base import MySQLIdentifierPreparer as MySQLIdentifierPreparer
from .base import TEXT as TEXT
from ... import sql as sql
from ... import util as util

class MySQLExecutionContext_mysqldb(MySQLExecutionContext):
    @property
    def rowcount(self): ...

class MySQLCompiler_mysqldb(MySQLCompiler): ...
class MySQLIdentifierPreparer_mysqldb(MySQLIdentifierPreparer): ...

class MySQLDialect_mysqldb(MySQLDialect):
    driver: str = ...
    supports_unicode_statements: bool = ...
    supports_sane_rowcount: bool = ...
    supports_sane_multi_rowcount: bool = ...
    supports_native_decimal: bool = ...
    default_paramstyle: str = ...
    execution_ctx_cls: Any = ...
    statement_compiler: Any = ...
    preparer: Any = ...
    def __init__(self, **kwargs: Any) -> None: ...
    def supports_server_side_cursors(self): ...
    @classmethod
    def dbapi(cls): ...
    def on_connect(self): ...
    def do_ping(self, dbapi_connection: Any): ...
    def do_executemany(
        self,
        cursor: Any,
        statement: Any,
        parameters: Any,
        context: Optional[Any] = ...,
    ) -> None: ...
    def create_connect_args(
        self, url: Any, _translate_args: Optional[Any] = ...
    ): ...

dialect = MySQLDialect_mysqldb



from datetime import date
from datetime import datetime
from datetime import time
from datetime import timedelta
from decimal import Decimal
from enum import Enum as _Enum
from typing import Any
from typing import ClassVar
from typing import Dict
from typing import Generic
from typing import List
from typing import Mapping
from typing import Optional
from typing import overload
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union

from typing_extensions import Literal

from .base import SchemaEventTarget
from .elements import BinaryExpression
from .traversals import HasCacheKey
from .type_api import _BindProcessor
from .type_api import _LiteralProcessor
from .type_api import _ResultProcessor
from .type_api import Emulated
from .type_api import TypeDecorator
from .type_api import TypeEngine
from .. import util
from ..engine import Connection
from ..engine import Dialect
from ..engine import Engine
from ..util import langhelpers

_T = TypeVar("_T")
_TE = TypeVar("_TE", bound=TypeEngine[Any])
_ST = TypeVar("_ST", bound=SchemaType)
_EN = TypeVar("_EN", bound=Enum)

_NumericType = Union[float, Decimal]
_JSONType = Union[str, float, bool, Mapping[Any, Any], List[Any], None]

class _LookupExpressionAdapter(Generic[_T]):
    class Comparator(TypeEngine.Comparator[_TE]): ...
    comparator_factory: Type[Comparator[TypeEngine[_T]]] = ...

class Concatenable(Generic[_T]):
    class Comparator(TypeEngine.Comparator[_TE]): ...
    comparator_factory: Type[Comparator[TypeEngine[_T]]] = ...

class Indexable(Generic[_T]):
    class Comparator(TypeEngine.Comparator[_TE]):
        def __getitem__(self, index: Any) -> Any: ...
    comparator_factory: Type[Comparator[TypeEngine[_T]]] = ...

class String(Concatenable[str], TypeEngine[str]):  # type: ignore[misc]
    __visit_name__: str = ...
    RETURNS_UNICODE: ClassVar[langhelpers._symbol] = ...
    RETURNS_BYTES: ClassVar[langhelpers._symbol] = ...
    RETURNS_CONDITIONAL: ClassVar[langhelpers._symbol] = ...
    RETURNS_UNKNOWN: ClassVar[langhelpers._symbol] = ...
    length: Optional[int] = ...
    collation: Optional[str] = ...
    @overload
    def __init__(
        self,
        length: Optional[int] = ...,
        collation: Optional[str] = ...,
        convert_unicode: bool = ...,
        _warn_on_bytestring: bool = ...,
        _expect_unicode: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self,
        length: Optional[int],
        collation: Optional[str],
        convert_unicode: Literal["force"],
        unicode_error: str,
        _warn_on_bytestring: bool = ...,
        _expect_unicode: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self,
        length: Optional[int],
        *,
        collation: Optional[str] = ...,
        convert_unicode: Literal["force"],
        unicode_error: str,
        _warn_on_bytestring: bool = ...,
        _expect_unicode: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self,
        *,
        length: Optional[int] = ...,
        collation: Optional[str] = ...,
        convert_unicode: Literal["force"],
        unicode_error: str,
        _warn_on_bytestring: bool = ...,
        _expect_unicode: bool = ...,
    ) -> None: ...
    def literal_processor(
        self, dialect: Dialect
    ) -> _LiteralProcessor[str]: ...
    def bind_processor(
        self, dialect: Dialect
    ) -> Optional[_BindProcessor[str]]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> Optional[_ResultProcessor[str]]: ...
    @property
    def python_type(self) -> Union[Type[str], Type[util.text_type]]: ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...

class Text(String):
    __visit_name__: str = ...

class Unicode(String):
    __visit_name__: str = ...
    def __init__(self, length: Optional[Any] = ..., **kwargs: Any) -> None: ...

class UnicodeText(Text):
    __visit_name__: str = ...
    def __init__(self, length: Optional[Any] = ..., **kwargs: Any) -> None: ...

class Integer(_LookupExpressionAdapter[int], TypeEngine[int]):  # type: ignore[misc]
    __visit_name__: str = ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...
    @property
    def python_type(self) -> Type[int]: ...
    def literal_processor(
        self, dialect: Dialect
    ) -> _LiteralProcessor[int]: ...

class SmallInteger(Integer):
    __visit_name__: str = ...

class BigInteger(Integer):
    __visit_name__: str = ...

class Numeric(  # type: ignore[misc]
    _LookupExpressionAdapter[_NumericType], TypeEngine[_NumericType]
):
    __visit_name__: str = ...
    precision: Any = ...
    scale: Any = ...
    decimal_return_scale: Any = ...
    asdecimal: Any = ...
    def __init__(
        self,
        precision: Optional[Any] = ...,
        scale: Optional[Any] = ...,
        decimal_return_scale: Optional[Any] = ...,
        asdecimal: bool = ...,
    ) -> None: ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...
    def literal_processor(
        self, dialect: Dialect
    ) -> _LiteralProcessor[_NumericType]: ...
    @property
    def python_type(self) -> Union[Type[float], Type[Decimal]]: ...
    def bind_processor(
        self, dialect: Dialect
    ) -> Optional[_BindProcessor[_NumericType]]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> Optional[_ResultProcessor[_NumericType]]: ...

class Float(Numeric):
    __visit_name__: str = ...
    scale: Any = ...
    precision: Any = ...
    asdecimal: Any = ...
    decimal_return_scale: Any = ...
    def __init__(
        self,
        precision: Optional[Any] = ...,
        asdecimal: bool = ...,
        decimal_return_scale: Optional[Any] = ...,
    ) -> None: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> Optional[_ResultProcessor[_NumericType]]: ...

class DateTime(_LookupExpressionAdapter[datetime], TypeEngine[datetime]):  # type: ignore[misc]
    __visit_name__: str = ...
    timezone: Any = ...
    def __init__(self, timezone: bool = ...) -> None: ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...
    @property
    def python_type(self) -> Type[datetime]: ...

class Date(_LookupExpressionAdapter[date], TypeEngine[date]):  # type: ignore[misc]
    __visit_name__: str = ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...
    @property
    def python_type(self) -> Type[date]: ...

class Time(_LookupExpressionAdapter[time], TypeEngine[time]):  # type: ignore[misc]
    __visit_name__: str = ...
    timezone: Any = ...
    def __init__(self, timezone: bool = ...) -> None: ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...
    @property
    def python_type(self) -> Type[time]: ...

class _Binary(TypeEngine[util.binary_type]):
    length: Any = ...
    def __init__(self, length: Optional[Any] = ...) -> None: ...
    def literal_processor(
        self, dialect: Dialect
    ) -> _LiteralProcessor[util.binary_type]: ...
    @property
    def python_type(self) -> Type[util.binary_type]: ...
    def bind_processor(
        self, dialect: Dialect
    ) -> Optional[_BindProcessor[util.binary_type]]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> _ResultProcessor[util.binary_type]: ...
    def coerce_compared_value(
        self, op: Any, value: Any
    ) -> TypeEngine[Any]: ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...

class LargeBinary(_Binary):
    __visit_name__: str = ...
    def __init__(self, length: Optional[Any] = ...) -> None: ...

class SchemaType(SchemaEventTarget):
    name: Any = ...
    schema: Any = ...
    metadata: Any = ...
    inherit_schema: Any = ...
    def __init__(
        self,
        name: Optional[Any] = ...,
        schema: Optional[Any] = ...,
        metadata: Optional[Any] = ...,
        inherit_schema: bool = ...,
        quote: Optional[Any] = ...,
        _create_events: bool = ...,
    ) -> None: ...
    def copy(self: _ST, **kw: Any) -> _ST: ...
    def adapt(self, __impltype: Type[_T], **kw: Any) -> _T: ...
    @property
    def bind(self) -> Optional[Union[Engine, Connection]]: ...
    def create(
        self,
        bind: Optional[Union[Engine, Connection]] = ...,
        checkfirst: bool = ...,
    ) -> None: ...
    def drop(
        self,
        bind: Optional[Union[Engine, Connection]] = ...,
        checkfirst: bool = ...,
    ) -> None: ...

class Enum(Emulated, String, SchemaType):
    __visit_name__: str = ...
    def __init__(self, *enums: Any, **kw: Any) -> None: ...
    @property
    def sort_key_function(self) -> util._SortKeyFunction: ...  # type: ignore[override]
    @property
    def native(self) -> Any: ...
    class Comparator(String.Comparator["Enum"]): ...
    comparator_factory: Type[Comparator] = ...  # type: ignore[assignment]
    def as_generic(self: _EN, allow_nulltype: bool = ...) -> _EN: ...
    def adapt_to_emulated(self, impltype: Type[_T], **kw: Any) -> _T: ...
    def adapt(self, __impltype: Type[_T], **kw: Any) -> _T: ...
    def literal_processor(
        self, dialect: Dialect
    ) -> _LiteralProcessor[str]: ...
    def bind_processor(self, dialect: Dialect) -> _BindProcessor[str]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> _ResultProcessor[str]: ...
    def copy(self: _EN, **kw: Any) -> _EN: ...
    @property
    def python_type(self) -> Union[Type[_Enum], Type[str], Type[util.text_type]]: ...  # type: ignore[override]

class PickleType(TypeDecorator[Any]):
    impl: Any = ...
    protocol: Any = ...
    pickler: Any = ...
    comparator: Any = ...
    def __init__(
        self,
        protocol: Any = ...,
        pickler: Optional[Any] = ...,
        comparator: Optional[Any] = ...,
    ) -> None: ...
    def __reduce__(self) -> Any: ...
    def bind_processor(self, dialect: Dialect) -> _BindProcessor[Any]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> _ResultProcessor[Any]: ...
    def compare_values(self, x: Any, y: Any) -> bool: ...

class Boolean(Emulated, TypeEngine[bool], SchemaType):  # type: ignore[misc]
    __visit_name__: str = ...
    native: bool = ...
    create_constraint: Any = ...
    name: Any = ...
    def __init__(
        self,
        create_constraint: bool = ...,
        name: Optional[Any] = ...,
        _create_events: bool = ...,
    ) -> None: ...
    @property
    def python_type(self) -> Type[bool]: ...
    def literal_processor(
        self, dialect: Dialect
    ) -> _LiteralProcessor[bool]: ...
    def bind_processor(self, dialect: Dialect) -> _BindProcessor[bool]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> _ResultProcessor[bool]: ...

class _AbstractInterval(_LookupExpressionAdapter[_T], TypeEngine[_T]):  # type: ignore[misc]
    def coerce_compared_value(
        self, op: Any, value: Any
    ) -> TypeEngine[Any]: ...

# "comparator_factory" of "_LookupExpressionAdapter" and "TypeDecorator" are incompatible
class Interval(Emulated, _AbstractInterval[timedelta], TypeDecorator[timedelta]):  # type: ignore[misc]
    impl: Any = ...
    epoch: datetime = ...
    native: bool = ...
    second_precision: Optional[float] = ...
    day_precision: Optional[float] = ...
    def __init__(
        self,
        native: bool = ...,
        second_precision: Optional[float] = ...,
        day_precision: Optional[float] = ...,
    ) -> None: ...
    @property
    def python_type(self) -> Type[timedelta]: ...
    def adapt_to_emulated(self, impltype: Type[_T], **kw: Any) -> _T: ...
    def bind_processor(
        self, dialect: Dialect
    ) -> _BindProcessor[timedelta]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> _ResultProcessor[timedelta]: ...

class JSON(
    Indexable[_JSONType],
    TypeEngine[_JSONType],
):
    __visit_name__: str = ...
    hashable: bool = ...
    NULL: Any = ...
    none_as_null: bool = ...
    should_evaluate_none: bool = ...
    def __init__(self, none_as_null: bool = ...) -> None: ...
    class JSONElementType(TypeEngine[Union[int, util.text_type]]):
        def string_bind_processor(
            self, dialect: Dialect
        ) -> Optional[_BindProcessor[util.text_type]]: ...
        def string_literal_processor(
            self, dialect: Dialect
        ) -> Optional[_LiteralProcessor[util.text_type]]: ...
        def bind_processor(
            self, dialect: Dialect
        ) -> _BindProcessor[Union[int, util.text_type]]: ...
        def literal_processor(
            self, dialect: Dialect
        ) -> _LiteralProcessor[Union[int, util.text_type]]: ...
    class JSONIndexType(JSONElementType): ...
    class JSONIntIndexType(JSONIndexType): ...
    class JSONStrIndexType(JSONIndexType): ...
    class JSONPathType(JSONElementType): ...
    class Comparator(
        Indexable.Comparator["JSON"], Concatenable.Comparator["JSON"]
    ):
        def as_boolean(self) -> Any: ...
        def as_string(self) -> Any: ...
        def as_integer(self) -> Any: ...
        def as_float(self) -> Any: ...
        def as_numeric(
            self, precision: Any, scale: Any, asdecimal: bool = ...
        ) -> Any: ...
        def as_json(self) -> Any: ...
    comparator_factory: Type[Comparator] = ...  # type: ignore[assignment]
    @property
    def python_type(self) -> Type[Dict[Any, Any]]: ...
    def bind_processor(
        self, dialect: Dialect
    ) -> _BindProcessor[_JSONType]: ...
    def result_processor(
        self, dialect: Dialect, coltype: Any
    ) -> _ResultProcessor[_JSONType]: ...

class ARRAY(
    SchemaEventTarget,
    Indexable[List[_TE]],
    Concatenable[List[_TE]],
    TypeEngine[List[_TE]],
):
    __visit_name__: str = ...
    class Comparator(
        Indexable.Comparator["ARRAY[_TE]"],
        Concatenable.Comparator["ARRAY[_TE]"],
    ):
        def contains(self, *arg: Any, **kw: Any) -> None: ...  # type: ignore[override]
        def any(
            self, other: Any, operator: Optional[Any] = ...
        ) -> BinaryExpression[Any]: ...
        def all(
            self, other: Any, operator: Optional[Any] = ...
        ) -> BinaryExpression[Any]: ...
    comparator_factory: Type[Comparator[_TE]] = ...  # type: ignore[assignment]
    item_type: _TE = ...
    as_tuple: bool = ...
    dimensions: Optional[int] = ...
    zero_indexes: bool = ...
    def __init__(
        self,
        item_type: Union[_TE, Type[_TE]],
        as_tuple: bool = ...,
        dimensions: Optional[int] = ...,
        zero_indexes: bool = ...,
    ) -> None: ...
    @property
    def hashable(self) -> bool: ...  # type: ignore[override]
    @property
    def python_type(self) -> Type[List[Any]]: ...
    def compare_values(self, x: Any, y: Any) -> bool: ...

class TupleType(TypeEngine[_TE]):
    types: Tuple[_TE, ...] = ...
    def __init__(self, *types: _TE) -> None: ...

class REAL(Float):
    __visit_name__: str = ...

class FLOAT(Float):
    __visit_name__: str = ...

class NUMERIC(Numeric):
    __visit_name__: str = ...

class DECIMAL(Numeric):
    __visit_name__: str = ...

class INTEGER(Integer):
    __visit_name__: str = ...

INT = INTEGER

class SMALLINT(SmallInteger):
    __visit_name__: str = ...

class BIGINT(BigInteger):
    __visit_name__: str = ...

class TIMESTAMP(DateTime):
    __visit_name__: str = ...
    def __init__(self, timezone: bool = ...) -> None: ...
    def get_dbapi_type(self, dbapi: Any) -> Any: ...

class DATETIME(DateTime):
    __visit_name__: str = ...

class DATE(Date):
    __visit_name__: str = ...

class TIME(Time):
    __visit_name__: str = ...

class TEXT(Text):
    __visit_name__: str = ...

class CLOB(Text):
    __visit_name__: str = ...

class VARCHAR(String):
    __visit_name__: str = ...

class NVARCHAR(Unicode):
    __visit_name__: str = ...

class CHAR(String):
    __visit_name__: str = ...

class NCHAR(Unicode):
    __visit_name__: str = ...

class BLOB(LargeBinary):
    __visit_name__: str = ...

class BINARY(_Binary):
    __visit_name__: str = ...

class VARBINARY(_Binary):
    __visit_name__: str = ...

class BOOLEAN(Boolean):
    __visit_name__: str = ...

class NullType(TypeEngine[None]):
    __visit_name__: str = ...
    hashable: bool = ...
    def literal_processor(
        self, dialect: Dialect
    ) -> _LiteralProcessor[None]: ...
    class Comparator(TypeEngine.Comparator["NullType"]): ...
    comparator_factory: Type[Comparator] = ...  # type: ignore[assignment]

class TableValueType(HasCacheKey, TypeEngine[Any]):
    def __init__(self, *elements: Any) -> None: ...

class MatchType(Boolean): ...

NULLTYPE: NullType
BOOLEANTYPE: Boolean
STRINGTYPE: String
INTEGERTYPE: Integer
MATCHTYPE: MatchType
TABLEVALUE: TableValueType



# fmt: off
from typing import Any
from typing import Optional

from sqlalchemy import processors as processors
from sqlalchemy import types as sqltypes
from sqlalchemy.dialects.sybase.base import SybaseDialect as SybaseDialect
from sqlalchemy.dialects.sybase.base import SybaseExecutionContext as SybaseExecutionContext
from sqlalchemy.dialects.sybase.base import SybaseSQLCompiler as SybaseSQLCompiler
# fmt: on

class _SybNumeric(sqltypes.Numeric):
    def result_processor(self, dialect: Any, type_: Any): ...

class SybaseExecutionContext_pysybase(SybaseExecutionContext):
    def set_ddl_autocommit(
        self, dbapi_connection: Any, value: Any
    ) -> None: ...
    def pre_exec(self) -> None: ...

class SybaseSQLCompiler_pysybase(SybaseSQLCompiler):
    def bindparam_string(self, name: Any, **kw: Any): ...

class SybaseDialect_pysybase(SybaseDialect):
    driver: str = ...
    execution_ctx_cls: Any = ...
    statement_compiler: Any = ...
    colspecs: Any = ...
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(self, url: Any): ...
    def do_executemany(
        self,
        cursor: Any,
        statement: Any,
        parameters: Any,
        context: Optional[Any] = ...,
    ) -> None: ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...

dialect = SybaseDialect_pysybase



from typing import Any
from typing import Callable
from typing import Generator
from typing import Mapping
from typing import NoReturn
from typing import Optional
from typing import TypeVar

from .base import ProxyComparable
from .base import StartableContext
from .result import AsyncResult
from ...engine import Result
from ...engine import Transaction
from ...future import Connection
from ...future import Engine
from ...sql import Executable

_TAsyncConnection = TypeVar("_TAsyncConnection", bound=AsyncConnection)
_TAsyncTransaction = TypeVar("_TAsyncTransaction", bound=AsyncTransaction)

def create_async_engine(*arg: Any, **kw: Any) -> AsyncEngine: ...

class AsyncConnectable: ...

class AsyncConnection(ProxyComparable, StartableContext, AsyncConnectable):
    # copied from future.Connection via create_proxy_methods
    @property
    def closed(self) -> bool: ...
    @property
    def invalidated(self) -> bool: ...
    dialect: Any
    @property
    def default_isolation_level(self) -> Any: ...
    # end copied

    engine: AsyncEngine = ...
    sync_engine: Engine = ...
    sync_connection: Optional[Connection] = ...
    def __init__(
        self,
        async_engine: AsyncEngine,
        sync_connection: Optional[Connection] = ...,
    ) -> None: ...
    async def start(self: _TAsyncConnection) -> _TAsyncConnection: ...
    @property
    def connection(self) -> NoReturn: ...
    async def get_raw_connection(self) -> Any: ...
    @property
    def info(self) -> Mapping[Any, Any]: ...
    def begin(self) -> AsyncTransaction: ...
    def begin_nested(self) -> AsyncTransaction: ...
    async def invalidate(self, exception: Optional[Any] = ...) -> None: ...
    async def get_isolation_level(self) -> Any: ...
    async def set_isolation_level(self) -> Any: ...
    def in_transaction(self) -> bool: ...
    def in_nested_transaction(self) -> bool: ...
    def get_transaction(self) -> Optional[AsyncTransaction]: ...
    def get_nested_transaction(self) -> Optional[AsyncTransaction]: ...
    async def execution_options(
        self: _TAsyncConnection, **opt: Any
    ) -> _TAsyncConnection: ...
    async def commit(self) -> None: ...
    async def rollback(self) -> None: ...
    async def close(self) -> None: ...
    async def exec_driver_sql(
        self,
        statement: str,
        parameters: Optional[Mapping[Any, Any]] = ...,
        execution_options: Mapping[Any, Any] = ...,
    ) -> Result: ...
    async def stream(
        self,
        statement: Executable,
        parameters: Optional[Mapping[Any, Any]] = ...,
        execution_options: Mapping[Any, Any] = ...,
    ) -> AsyncResult: ...
    async def execute(
        self,
        statement: Executable,
        parameters: Optional[Mapping[Any, Any]] = ...,
        execution_options: Mapping[Any, Any] = ...,
    ) -> Result: ...
    async def scalar(
        self,
        statement: Executable,
        parameters: Optional[Mapping[Any, Any]] = ...,
        execution_options: Mapping[Any, Any] = ...,
    ) -> Any: ...
    async def run_sync(
        self, fn: Callable[..., Any], *arg: Any, **kw: Any
    ) -> Any: ...
    def __await__(
        self: _TAsyncConnection,
    ) -> Generator[Any, None, _TAsyncConnection]: ...
    async def __aexit__(
        self, type_: Any, value: Any, traceback: Any
    ) -> None: ...

class AsyncEngine(ProxyComparable, AsyncConnectable):
    # copied from future.Engine by create_proxy_methods
    def clear_compiled_cache(self) -> None: ...
    def update_execution_options(self, **opt: Any) -> None: ...
    def get_execution_options(self) -> Mapping[Any, Any]: ...
    # end copied
    class _trans_ctx(StartableContext):
        conn: AsyncConnection = ...
        def __init__(self, conn: AsyncConnection) -> None: ...
        transaction: Any = ...
        async def start(self) -> AsyncConnection: ...  # type: ignore[override]
        def __await__(self) -> Generator[Any, None, AsyncConnection]: ...  # type: ignore[override]
        async def __aexit__(
            self, type_: Any, value: Any, traceback: Any
        ) -> None: ...
    sync_engine: Engine = ...
    def __init__(self, sync_engine: Engine) -> None: ...
    def begin(self) -> _trans_ctx: ...
    def connect(self) -> AsyncConnection: ...
    async def raw_connection(self) -> Any: ...
    def execution_options(self, **opt: Any) -> AsyncEngine: ...
    async def dispose(self) -> None: ...

class AsyncTransaction(ProxyComparable, StartableContext):
    connection: AsyncConnection = ...
    sync_transaction: Optional[Transaction] = ...
    nested: bool = ...
    def __init__(
        self, connection: AsyncConnection, nested: bool = ...
    ) -> None: ...
    @property
    def is_valid(self) -> bool: ...
    @property
    def is_active(self) -> bool: ...
    async def close(self) -> None: ...
    async def rollback(self) -> None: ...
    async def commit(self) -> None: ...
    async def start(self: _TAsyncTransaction) -> _TAsyncTransaction: ...
    async def __aexit__(
        self, type_: Any, value: Any, traceback: Any
    ) -> None: ...



from typing import Any

from .base import MySQLCompiler as MySQLCompiler
from .base import MySQLDialect as MySQLDialect
from .base import MySQLExecutionContext as MySQLExecutionContext
from .base import MySQLIdentifierPreparer as MySQLIdentifierPreparer
from ... import sql as sql
from ... import util as util

mariadb_cpy_minimum_version: Any

class MySQLExecutionContext_mariadbconnector(MySQLExecutionContext):
    def create_server_side_cursor(self): ...
    def create_default_cursor(self): ...

class MySQLCompiler_mariadbconnector(MySQLCompiler): ...
class MySQLIdentifierPreparer_mariadbconnector(MySQLIdentifierPreparer): ...

class MySQLDialect_mariadbconnector(MySQLDialect):
    driver: str = ...
    supports_unicode_statements: bool = ...
    encoding: str = ...
    convert_unicode: bool = ...
    supports_sane_rowcount: bool = ...
    supports_sane_multi_rowcount: bool = ...
    supports_native_decimal: bool = ...
    default_paramstyle: str = ...
    execution_ctx_cls: Any = ...
    statement_compiler: Any = ...
    preparer: Any = ...
    supports_server_side_cursors: bool = ...
    paramstyle: str = ...
    def __init__(self, **kwargs: Any) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...
    def create_connect_args(self, url: Any): ...
    def do_begin_twophase(self, connection: Any, xid: Any) -> None: ...
    def do_prepare_twophase(self, connection: Any, xid: Any) -> None: ...
    def do_rollback_twophase(
        self,
        connection: Any,
        xid: Any,
        is_prepared: bool = ...,
        recover: bool = ...,
    ) -> None: ...
    def do_commit_twophase(
        self,
        connection: Any,
        xid: Any,
        is_prepared: bool = ...,
        recover: bool = ...,
    ) -> None: ...

dialect = MySQLDialect_mariadbconnector



from typing import Any
from typing import Optional

from ..orm.query import Query
from ..orm.session import Session

class ShardedQuery(Query):
    id_chooser: Any = ...
    query_chooser: Any = ...
    execute_chooser: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def set_shard(self, shard_id: Any): ...

class ShardedSession(Session):
    shard_chooser: Any = ...
    id_chooser: Any = ...
    execute_chooser: Any = ...
    query_chooser: Any = ...
    def __init__(
        self,
        shard_chooser: Any,
        id_chooser: Any,
        execute_chooser: Optional[Any] = ...,
        shards: Optional[Any] = ...,
        query_cls: Any = ...,
        **kwargs: Any,
    ): ...
    def connection_callable(
        self,
        mapper: Optional[Any] = ...,
        instance: Optional[Any] = ...,
        shard_id: Optional[Any] = ...,
        **kwargs: Any,
    ): ...
    def get_bind(
        self,
        mapper: Optional[Any] = ...,
        shard_id: Optional[Any] = ...,
        instance: Optional[Any] = ...,
        clause: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def bind_shard(self, shard_id: Any, bind: Any) -> None: ...



from typing import Any
from typing import Optional

from .. import util as util

class _UnpickleDispatch:
    def __call__(self, _instance_cls: Any): ...

class _Dispatch:
    def __init__(
        self, parent: Any, instance_cls: Optional[Any] = ...
    ) -> None: ...
    def __getattr__(self, name: Any): ...
    def __reduce__(self): ...

class _EventMeta(type):
    def __init__(cls, classname: Any, bases: Any, dict_: Any) -> None: ...

class Events: ...

class _JoinedDispatcher:
    local: Any = ...
    parent: Any = ...
    def __init__(self, local: Any, parent: Any) -> None: ...
    def __getattr__(self, name: Any): ...

class dispatcher:
    dispatch: Any = ...
    events: Any = ...
    def __init__(self, events: Any) -> None: ...
    def __get__(self, obj: Any, cls: Any): ...

class slots_dispatcher(dispatcher):
    def __get__(self, obj: Any, cls: Any): ...
